#!/usr/bin/env python
#############################################################################
# Copyright (c) 2020 One Identity
# Copyright (c) 2021 Xiaoyu Qiu
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import pytest


test_parameters_raw = [
    {
        "id": "simple",
        "input_message": "foo,bar",
        "expected_value": "foo=foo bar=bar",
    },
    {
        "id": "quoted",
        "input_message": '''foo,"bar"''',
        "expected_value": "foo=foo bar=bar",
    },
    {
        "id": "simple-quotes",
        "quotes": "~^",
        "input_message": '''~foo~,^bar^''',
        "expected_value": "foo=foo bar=bar",
    },
    {
        "id": "quote-pairs",
        "quote-pairs": "><~~",
        "input_message": '''~foo~,>bar<''',
        "expected_value": "foo=foo bar=bar",
    },
    {
        "id": "escape-none",
        "input_message": "foo,bar",
        "dialect": "escape-none",
        "expected_value": "foo=foo bar=bar",
    },
    {
        "id": "escape-double-char",
        "input_message": '''foo,"b""a""r"''',
        "dialect": "escape-double-char",
        "expected_value": 'foo=foo bar=b"a"r',
    },
    {
        "id": "escape-backslash",
        "input_message": r'''foo,"b\"a\"r\a"''',
        "dialect": "escape-backslash",
        "expected_value": 'foo=foo bar=b"a"ra',
    },
    {
        "id": "escape-backslash-with-sequences",
        "input_message": r'''foo,"b\"a\"r\a"''',
        "dialect": "escape-backslash-with-sequences",
        "expected_value": 'foo=foo bar=b"a"r\a',
    },
    {
        "id": "null-value",
        "input_message": "foo,NULL",
        "null": "NULL",
        "expected_value": "foo=foo bar=>>unset<<",
    },
    {
        "id": "delimiter-character",
        "input_message": "foo^bar",
        "delimiters": '"^"',
        "expected_value": "foo=foo bar=bar",
    },
    {
        "id": "delimiter-character2",
        "input_message": "foo^bar",
        "delimiters": 'chars("^")',
        "expected_value": "foo=foo bar=bar",
    },
    {
        "id": "delimiter-string",
        "input_message": "foo^^^bar",
        "delimiters": 'strings("^^^", "~~~")',
        "expected_value": "foo=foo bar=bar",
    },
    {
        "id": "delimiter-string2",
        "input_message": "foo~~~bar",
        "delimiters": 'strings("^^^", "~~~")',
        "expected_value": "foo=foo bar=bar",
    },
]


def add_optional_arg(config, args, testcase, key):
    if key in testcase:
        args[key] = testcase[key]


def add_optional_string_arg(config, args, testcase, key):
    if key in testcase:
        args[key] = config.stringify(testcase[key])


@pytest.mark.parametrize(
    "testcase", test_parameters_raw,
    ids=map(lambda tc: tc['id'], test_parameters_raw),
)
def test_csv_parser(config, syslog_ng, testcase):
    config.update_global_options(stats_level=1)
    generator_source = config.create_example_msg_generator_source(num=1, template=config.stringify(testcase["input_message"]))
    csv_parser_args = {}

    csv_parser_args['delimiters'] = config.stringify(',')
    add_optional_arg(config, csv_parser_args, testcase, 'dialect')
    add_optional_string_arg(config, csv_parser_args, testcase, 'quotes')
    add_optional_string_arg(config, csv_parser_args, testcase, 'quote-pairs')
    add_optional_string_arg(config, csv_parser_args, testcase, 'null')
    add_optional_arg(config, csv_parser_args, testcase, 'delimiters')
    csv_parser = config.create_csv_parser(prefix=config.stringify("prefix."), columns='''"foo", "bar"''', **csv_parser_args)

    file_destination = config.create_file_destination(file_name="output.log", template=config.stringify("foo=${prefix.foo:->>unset<<} bar=${prefix.bar:->>unset<<}\n"))
    config.create_logpath(statements=[generator_source, csv_parser, file_destination])

    syslog_ng.start(config)

    assert file_destination.read_log().strip() == testcase['expected_value']
    assert csv_parser.get_query().get('discarded', -1) == 0


def test_csv_parser_into_matches(config, syslog_ng):
    config.update_global_options(stats_level=1)
    generator_source = config.create_example_msg_generator_source(num=1, template=config.stringify("FOO,BAR,BAZ"))

    csv_parser = config.create_csv_parser(delimiters=config.stringify(","))

    file_destination = config.create_file_destination(file_name="output.log", template=config.stringify("$*\n"))
    config.create_logpath(statements=[generator_source, csv_parser, file_destination])

    syslog_ng.start(config)

    assert file_destination.read_log().strip() == "FOO,BAR,BAZ"
    assert csv_parser.get_query().get('discarded', -1) == 0


test_parameters_drop_invalid = [
    {
        "id": "too_many_columns_in_input",
        "input_message": "foo,bar,baz",
        "columns": "foo,bar",
    },
    {
        "id": "type_does_not_match",
        "input_message": "foo,bar",
        "columns": "int(foo),bar",
    },
]


@pytest.mark.parametrize(
    "testcase", test_parameters_drop_invalid,
    ids=map(lambda tc: tc['id'], test_parameters_drop_invalid),
)
def test_csv_parser_drop_invalid(config, syslog_ng, testcase):
    config.update_global_options(stats_level=1)
    generator_source = config.create_example_msg_generator_source(num=1, template=config.stringify(testcase["input_message"]))

    csv_parser = config.create_csv_parser(delimiters=config.stringify(","), drop_invalid="yes", columns=testcase["columns"])

    file_destination = config.create_file_destination(file_name="output.log", template=config.stringify("$*\n"))
    config.create_logpath(statements=[generator_source, csv_parser, file_destination])

    syslog_ng.start(config)

    assert csv_parser.get_query().get('discarded', -1) == 1



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2023 Attila Szakacs
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from . import assert_multiple_logpath_stats
from . import ExpectedLogPathStats


def test_named_logpaths_with_fallback_flag(config, port_allocator, syslog_ng, log_message, bsd_formatter):
    config.update_global_options(stats_level=1)

    source = config.create_statement_group(statements=[config.create_network_source(port=port_allocator())])

    destination_1 = config.create_statement_group(statements=[config.create_file_destination(file_name="output-1.log")])
    destination_2 = config.create_statement_group(statements=[config.create_file_destination(file_name="output-2.log")])

    filter_top_level_1 = config.create_statement_group(statements=[config.create_filter(message="'to-top-level-1'")])
    filter_top_level_2 = config.create_statement_group(statements=[config.create_filter(message="'to-top-level-2'")])
    filter_top_level_3 = config.create_statement_group(statements=[config.create_filter(message="'to-top-level-3'")])
    filter_always_false = config.create_statement_group(statements=[config.create_filter("1 == 2")])
    filter_always_true = config.create_statement_group(statements=[config.create_filter("1 == 1")])

    # log top-level-1 {
    #     source(s);
    #     filter(f_t1);
    #     log inner-1 { filter(f_false); destination(d_1); };
    #     log inner-fallback-2 { destination(d_2); flags(fallback); };
    # };
    inner_1 = config.create_inner_logpath(
        name="inner-1",
        statements=[
            filter_always_false,
            destination_1,
        ],
    )
    inner_fallback_2 = config.create_inner_logpath(
        name="inner-fallback-2",
        statements=[
            destination_2,
        ],
        flags="fallback",
    )
    top_level_1 = config.create_logpath(
        name="top-level-1",
        statements=[
            source,
            filter_top_level_1,
            inner_1,
            inner_fallback_2,
        ],
    )

    # log top-level-2 {
    #     source(s);
    #     filter(f_t2);
    #     log inner-3 { filter(f_true); destination(d_1); };
    #     log inner-fallback-4 { destination(d_2); flags(fallback); };
    # };
    inner_3 = config.create_inner_logpath(
        name="inner-3",
        statements=[
            filter_always_true,
            destination_1,
        ],
    )
    inner_fallback_4 = config.create_inner_logpath(
        name="inner-fallback-4",
        statements=[
            destination_2,
        ],
        flags="fallback",
    )
    top_level_2 = config.create_logpath(
        name="top-level-2",
        statements=[
            source,
            filter_top_level_2,
            inner_3,
            inner_fallback_4,
        ],
    )

    # log top-level-3 {
    #     source(s);
    #     filter(f_t3);
    #     log inner-5 { filter(f_false); destination(d_1); };
    #     log inner-fallback-6 { destination(d_2); flags(fallback); };
    # };
    inner_5 = config.create_inner_logpath(
        name="inner-5",
        statements=[
            filter_always_false,
            destination_1,
        ],
    )
    inner_fallback_6 = config.create_inner_logpath(
        name="inner-fallback-6",
        statements=[
            destination_2,
        ],
        flags="fallback",
    )
    top_level_3 = config.create_logpath(
        name="top-level-3",
        statements=[
            source,
            filter_top_level_3,
            destination_1,
            inner_5,
            inner_fallback_6,
        ],
    )

    logpaths = {
        "top-level-1": ExpectedLogPathStats(top_level_1, 0, 0),
        "inner-1": ExpectedLogPathStats(inner_1, 0, 0),
        "inner-fallback-2": ExpectedLogPathStats(inner_fallback_2, 0, 0),
        "top-level-2": ExpectedLogPathStats(top_level_2, 0, 0),
        "inner-3": ExpectedLogPathStats(inner_3, 0, 0),
        "inner-fallback-4": ExpectedLogPathStats(inner_fallback_4, 0, 0),
        "top-level-3": ExpectedLogPathStats(top_level_3, 0, 0),
        "inner-5": ExpectedLogPathStats(inner_5, 0, 0),
        "inner-fallback-6": ExpectedLogPathStats(inner_fallback_6, 0, 0),
    }

    syslog_ng.start(config)
    assert_multiple_logpath_stats(logpaths)

    source[0].write_log(bsd_formatter.format_message(log_message.message("to-top-level-1")))
    logpaths["top-level-1"].ingress += 1
    logpaths["top-level-1"].egress += 1
    logpaths["inner-1"].ingress += 1
    logpaths["top-level-2"].ingress += 1
    logpaths["inner-fallback-2"].ingress += 1
    logpaths["inner-fallback-2"].egress += 1
    logpaths["top-level-3"].ingress += 1
    destination_2[0].read_until_logs(["to-top-level-1"])
    assert_multiple_logpath_stats(logpaths)

    source[0].write_log(bsd_formatter.format_message(log_message.message("to-top-level-2")))
    logpaths["top-level-1"].ingress += 1
    logpaths["top-level-2"].ingress += 1
    logpaths["top-level-2"].egress += 1
    logpaths["inner-3"].ingress += 1
    logpaths["inner-3"].egress += 1
    logpaths["top-level-3"].ingress += 1
    destination_1[0].read_until_logs(["to-top-level-2"])
    assert_multiple_logpath_stats(logpaths)

    source[0].write_log(bsd_formatter.format_message(log_message.message("to-top-level-3")))
    logpaths["top-level-1"].ingress += 1
    logpaths["top-level-2"].ingress += 1
    logpaths["top-level-3"].ingress += 1
    logpaths["top-level-3"].egress += 1
    logpaths["inner-5"].ingress += 1
    logpaths["inner-fallback-6"].ingress += 1
    logpaths["inner-fallback-6"].egress += 1
    destination_1[0].read_until_logs(["to-top-level-3"])
    assert_multiple_logpath_stats(logpaths)



#############################################################################
# Copyright (c) 2015-2016 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

class LogDestination(object):

    def open(self):
        """Open a connection to the target service"""
        return True

    def close(self):
        """Close the connection to the target service"""
        pass

    def is_opened(self):
        """Check if the connection to the target is able to receive messages"""
        return True

    def init(self, options):
        """This method is called at initialization time"""
        return True

    def deinit(self):
        """This method is called at deinitialization time"""
        pass

    def send(self, msg):
        """Send a message to the target service

        It can return either a boolean or integer.
        Boolean: True to indicate success, False will suspend the
        destination for a period specified by the time-reopen() option.
        After that the same message is retried until retries() times.

        Integer:
        self.SUCCESS: message sending was successful (same as boolean True)
        self.ERROR: message sending was unsuccessful. Same message is retried.
            (same as boolean False)
        self.DROP: message cannot be sent, it should be dropped immediately.
        self.QUEUED: message is not sent immediately, it will be sent with the flush method.
        self.NOT_CONNECTED: message is put back to the queue, open method will be called until success.
        self.RETRY: message is put back to the queue, try to send again until 3 times, then fallback to self.NOT_CONNECTED."""

        pass

    def flush(self):
        """Flush the queued messages

        It can return either a boolean or integer.
        Boolean: True to indicate that the batch is successfully sent.
        False indicates error while sending the batch. The destination is suspended
        for time-reopen period. The messages in the batch are passed again to send, one by one.

        Integer:
        self.SUCCESS: batch sending was successful (same as boolean True)
        self.ERROR: batch sending was unsuccessful. (same as boolean False)
        self.DROP: batch cannot be sent, the messages should be dropped immediately.
        self.NOT_CONNECTED: the messages in the batch is put back to the queue,
            open method will be called until success.
        self.RETRY: message is put back to the queue, try to send again until 3 times, then fallback to self.NOT_CONNECTED."""

        pass

class DummyPythonDest(object):
    def send(self, msg):
        print('queue', msg)
        return self.SUCCESS

class DummyBatchDestination(object):
    def init(self, options):
        self.bulk = list()
        return True

    def send(self, msg):
         self.bulk.append(msg["MSG"].decode())
         return self.QUEUED

    def flush(self):
        print("flushing: " + ",".join(self.bulk))
        self.bulk = list()
        return self.SUCCESS



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2023 Attila Szakacs
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from typing import Dict
from typing import NamedTuple
from typing import Optional

import prometheus_client.parser
from prometheus_client.samples import Sample

import src.testcase_parameters.testcase_parameters as tc_parameters
from src.syslog_ng_ctl.syslog_ng_ctl import SyslogNgCtl

__all__ = ["PrometheusStatsHandler", "MetricFilter", "Sample"]


class MetricFilter(NamedTuple):
    name: str
    labels: Optional[Dict[str, str]] = None


class PrometheusStatsHandler(object):
    def __init__(self, metric_filters):
        self.__metric_filters = metric_filters
        self.__syslog_ng_ctl = SyslogNgCtl(tc_parameters.INSTANCE_PATH)

    def __filter_raw_samples(self, raw_samples):
        samples = []

        for metric_family in prometheus_client.parser.text_string_to_metric_families(raw_samples):
            for sample in metric_family.samples:
                for metric_filter in self.__metric_filters:
                    if metric_filter.name == sample.name and metric_filter.labels.items() <= sample.labels.items():
                        samples.append(sample)

        return samples

    def get_samples(self):
        if len(self.__metric_filters) == 0:
            return []

        ctl_output = self.__syslog_ng_ctl.stats_prometheus()
        if ctl_output["exit_code"] != 0:
            return []

        return self.__filter_raw_samples(ctl_output["stdout"])



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2023 Attila Szakacs
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from . import assert_multiple_logpath_stats
from . import ExpectedLogPathStats


def test_named_logpaths_with_catchall_flag(config, port_allocator, syslog_ng, log_message, bsd_formatter):
    config.update_global_options(stats_level=1)

    source = config.create_statement_group(statements=[config.create_network_source(port=port_allocator())])

    destination = config.create_statement_group(statements=[config.create_file_destination(file_name="output.log")])

    filter_top_level_1 = config.create_statement_group(statements=[config.create_filter(message="'to-top-level-1'")])
    filter_top_level_2 = config.create_statement_group(statements=[config.create_filter(message="'to-top-level-2'")])

    # log top-level-1 {
    #     filter(f_t1);
    #     destination(d);
    #     flags(catchall);
    # };
    top_level_1 = config.create_logpath(
        name="top-level-1",
        statements=[
            filter_top_level_1,
            destination,
        ],
        flags="catchall",
    )

    # log top-level-2 {
    #     filter(f_t2);
    #     destination(d);
    #     flags(catchall);
    # };
    top_level_2 = config.create_logpath(
        name="top-level-2",
        statements=[
            filter_top_level_2,
            destination,
        ],
        flags="catchall",
    )

    logpaths = {
        "top-level-1": ExpectedLogPathStats(top_level_1, 0, 0),
        "top-level-2": ExpectedLogPathStats(top_level_2, 0, 0),
    }

    syslog_ng.start(config)
    assert_multiple_logpath_stats(logpaths)

    source[0].write_log(bsd_formatter.format_message(log_message.message("to-top-level-1")))
    logpaths["top-level-1"].ingress += 1
    logpaths["top-level-1"].egress += 1
    logpaths["top-level-2"].ingress += 1
    destination[0].read_until_logs(["to-top-level-1"])
    assert_multiple_logpath_stats(logpaths)

    source[0].write_log(bsd_formatter.format_message(log_message.message("to-top-level-2")))
    logpaths["top-level-1"].ingress += 1
    logpaths["top-level-2"].ingress += 1
    logpaths["top-level-2"].egress += 1
    destination[0].read_until_logs(["to-top-level-2"])
    assert_multiple_logpath_stats(logpaths)



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2020 One Identity
# Copyright (c) 2021 Xiaoyu Qiu
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import pytest


test_parameters_raw = [
    ("foo", '''""''', '''"(?<key>foo)"''', "", "${key}", True, True, "foo"),
    ("foo", '''""''', '''"(?<key>fo*)"''', "", "${key}", True, True, "foo"),
    ("foo", '''".reg."''', '''"(?<key>foo)"''', "", "${.reg.key}", True, True, "foo"),
    ("foo", '''".reg."''', '''"(?<key>foo)"''', "", "${key}", True, True, ""),
    ("foo", '''".reg."''', '''"(?<key>foo)|(?<key>bar)"''', "dupnames", "${.reg.key}", True, True, "foo"),
    ("abc", '''""''', '''"(?<key>Abc)"''', "", "${key}", True, False, ""),
    ("abc", '''""''', '''"(?<key>Abc)"''', "ignore-case", "${key}", True, True, "abc"),
    ("foobar", '''".reg."''', '''"(?<key>foo)", "(?<key>bar)"''', "", "${.reg.key}", True, True, "foo"),
    ("foo", '''""''', '''"(?<key>foo"''', "", "${key}", False, False, ""),
    ("foo", '''""''', '''"fo*"''', "", "$MSG", True, True, "foo"),
]


@pytest.mark.parametrize(
    "input_message, prefix, patterns, flags, template, compile_result, expected_result, expected_value", test_parameters_raw,
    ids=[
        "match_literally", "match_regular_expression",
        "match_with_prefix", "match_with_prefix_and_wrong_template", "match_with_dupnames", "unmatch_case",
        "match_ignore_case_flag", "match_multiple_regular_expressions", "regular_expression_compile_error",
        "check_MSG_macro",
    ],
)
def test_regexp_parser(config, syslog_ng, input_message, prefix, patterns, flags, template, compile_result, expected_result, expected_value):
    config.update_global_options(stats_level=1)
    generator_source = config.create_example_msg_generator_source(num=1, template=config.stringify(input_message))
    regexp_parser = config.create_regexp_parser(prefix=prefix, patterns=patterns, flags=flags)

    file_destination = config.create_file_destination(file_name="output.log", template=config.stringify(template + "\n"))
    config.create_logpath(statements=[generator_source, regexp_parser, file_destination])

    if compile_result:
        syslog_ng.start(config)
        if expected_result:
            assert file_destination.read_log().strip() == expected_value
            assert regexp_parser.get_query().get('discarded', -1) == 0
        else:
            assert regexp_parser.get_query().get('discarded', -1) == 1
    else:
        with pytest.raises(Exception):
            syslog_ng.start(config)



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2020 One Identity
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import socket
from enum import Enum
from enum import IntEnum
from pathlib import Path

from src.common.asynchronous import BackgroundEventLoop
from src.common.blocking import DEFAULT_TIMEOUT
from src.common.file import File
from src.common.network import SingleConnectionTCPServer
from src.common.network import UDPServer
from src.common.random_id import get_unique_id
from src.driver_io import message_readers
from src.helpers.loggen.loggen import Loggen


class NetworkIO():
    def __init__(self, ip, port, transport, ip_proto_version=None):
        self.__ip = ip
        self.__port = port
        self.__transport = transport
        self.__ip_proto_version = NetworkIO.IPProtoVersion.V4 if ip_proto_version is None else ip_proto_version
        self.__server = None
        self.__message_reader = None

    def write(self, content, rate=None):
        loggen_input_file_path = Path("loggen_input_{}.txt".format(get_unique_id()))

        loggen_input_file = File(loggen_input_file_path)
        loggen_input_file.write_content_and_close(content)

        Loggen().start(self.__ip, self.__port, read_file=str(loggen_input_file_path), dont_parse=True, permanent=True, rate=rate, **self.__transport.value)

    def start_listener(self):
        self.__message_reader, self.__server = self.__transport.construct(self.__port, self.__ip, self.__ip_proto_version)
        BackgroundEventLoop().wait_async_result(self.__server.start(), timeout=DEFAULT_TIMEOUT)

    def stop_listener(self):
        if self.__message_reader is not None:
            BackgroundEventLoop().wait_async_result(self.__server.stop(), timeout=DEFAULT_TIMEOUT)
            self.__message_reader = None
            self.__server = None

    def read_number_of_messages(self, counter, timeout=DEFAULT_TIMEOUT):
        return self.__message_reader.wait_for_number_of_messages(counter, timeout)

    def read_until_messages(self, lines, timeout=DEFAULT_TIMEOUT):
        return self.__message_reader.wait_for_messages(lines, timeout)

    class IPProtoVersion(IntEnum):
        V4 = socket.AF_INET
        V6 = socket.AF_INET6

    class Transport(Enum):
        TCP = {"inet": True, "stream": True}
        UDP = {"dgram": True}
        TLS = {"use_ssl": True}
        PROXIED_TCP = {"inet": True, "stream": True}
        PROXIED_TLS = {"use_ssl": True}
        PROXIED_TLS_PASSTHROUGH = {"use_ssl": True, "proxied_tls_passthrough": True}

        def construct(self, port, host=None, ip_proto_version=None):
            def _construct(server, reader_class):
                return reader_class(server), server

            transport_mapping = {
                NetworkIO.Transport.TCP: lambda: _construct(SingleConnectionTCPServer(port, host, ip_proto_version), message_readers.SingleLineStreamReader),
                NetworkIO.Transport.UDP: lambda: _construct(UDPServer(port, host, ip_proto_version), message_readers.DatagramReader),
            }
            return transport_mapping[self]()



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2023 Szilard Parrag
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import json

import pytest


test_parameters = [
    (
        "2023-09-20 16:50:02.165 CET,,,407764,,650b069a.638d4,2,,2023-09-20 16:50:02 CET,,0,DEBUG3,00000,\"starting PostgreSQL 15.4 (Ubuntu 15.4-0ubuntu0.23.04.1) on x86_64-pc-linux-gnu, compiled by gcc (Ubuntu 12.3.0-1ubuntu1~23.04) 12.3.0, 64-bit\",,,,,,,,,\"\",\"postmaster\",,0",
        {
            "DATE": "Sep 20 16:50:02",
            "HOST_FROM": "localhost",
            "MSG": "starting PostgreSQL 15.4 (Ubuntu 15.4-0ubuntu0.23.04.1) on x86_64-pc-linux-gnu, compiled by gcc (Ubuntu 12.3.0-1ubuntu1~23.04) 12.3.0, 64-bit",
            "PID": 407764,
            "SEVERITY": "debug",
            "_pgsql": {
                "application_name": "",
                "command_tag": "",
                "connection_from": "localhost",
                "context": "",
                "database": "",
                "detail": "",
                "hint": "",
                "internal_query": "",
                "location": "",
                "message": "starting PostgreSQL 15.4 (Ubuntu 15.4-0ubuntu0.23.04.1) on x86_64-pc-linux-gnu, compiled by gcc (Ubuntu 12.3.0-1ubuntu1~23.04) 12.3.0, 64-bit",
                "pid": 407764,
                "query": "",
                "session_id": "650b069a.638d4",
                "session_line_num": 2,
                "session_start_time": "2023-09-20 16:50:02 CET",
                "severity": "LOG",
                "sql_state_code": "00000",
                "timestamp": "2023-09-20 16:50:02.165 CET",
                "transaction_id": "0",
                "username": "",
                "virtual_transaction_id": "",
            },
        },
    ),
    (
        r'2023-08-08 12:05:52.805 UTC,,,22113,,64d22fa0.5661,1,,2023-08-08 12:05:52 UTC,23/74060,0,LOG,00000,"automatic vacuum of table ""tablename"": index scans: 0 pages: 0 removed, 4 remain, 0 skipped due to pins, 0 skipped frozen tuples: 114 removed, 268 remain, 0 are dead but not yet removable, oldest xmin: 149738000 buffer usage: 97 hits, 0 misses, 6 dirtied avg read rate: 0.000 MB/s, avg write rate: 114.609 MB/s system usage: CPU: user: 0.00 s, system: 0.00 s, elapsed: 0.00 s",,,,,,,,,""',
        {
            "DATE": "Aug  8 12:05:52",
            "HOST_FROM": "localhost",
            "MSG": r'automatic vacuum of table "tablename": index scans: 0 pages: 0 removed, 4 remain, 0 skipped due to pins, 0 skipped frozen tuples: 114 removed, 268 remain, 0 are dead but not yet removable, oldest xmin: 149738000 buffer usage: 97 hits, 0 misses, 6 dirtied avg read rate: 0.000 MB/s, avg write rate: 114.609 MB/s system usage: CPU: user: 0.00 s, system: 0.00 s, elapsed: 0.00 s',
            "PID": 22113,
            "SEVERITY": "info",
            "_pgsql": {
                "application_name": "",
                "command_tag": "",
                "connection_from": "localhost",
                "context": "",
                "database": "",
                "detail": "",
                "hint": "",
                "internal_query": "",
                "location": "",
                "message": r'automatic vacuum of table "tablename": index scans: 0 pages: 0 removed, 4 remain, 0 skipped due to pins, 0 skipped frozen tuples: 114 removed, 268 remain, 0 are dead but not yet removable, oldest xmin: 149738000 buffer usage: 97 hits, 0 misses, 6 dirtied avg read rate: 0.000 MB/s, avg write rate: 114.609 MB/s system usage: CPU: user: 0.00 s, system: 0.00 s, elapsed: 0.00 s',
                "pid": 22113,
                "query": "",
                "session_id": "64d22fa0.5661",
                "session_line_num": 1,
                "session_start_time": "2023-08-08 12:05:52 UTC",
                "severity": "LOG",
                "sql_state_code": "00000",
                "timestamp": "2023-08-08 12:05:52.805 UTC",
                "transaction_id": "0",
                "username": "",
                "virtual_transaction_id": "",
            },
        },
    ),
    (
        r'2023-08-08 12:05:52.805 UTC,,,22113,,64d22fa0.5661,1,,2023-08-08 12:05:52 UTC,23/74060,0,WARNING,00000,"automatic vacuum of table ""tablename"": index scans: 0 pages: 0 removed, 4 remain, 0 skipped due to pins, 0 skipped frozen tuples: 114 removed, 268 remain, 0 are dead but not yet removable, oldest xmin: 149738000 buffer usage: 97 hits, 0 misses, 6 dirtied avg read rate: 0.000 MB/s, avg write rate: 114.609 MB/s system usage: CPU: user: 0.00 s, system: 0.00 s, elapsed: 0.00 s",,,,,,,,,""',
        {
            "DATE": "Aug  8 12:05:52",
            "HOST_FROM": "localhost",
            "MSG": r'automatic vacuum of table "tablename": index scans: 0 pages: 0 removed, 4 remain, 0 skipped due to pins, 0 skipped frozen tuples: 114 removed, 268 remain, 0 are dead but not yet removable, oldest xmin: 149738000 buffer usage: 97 hits, 0 misses, 6 dirtied avg read rate: 0.000 MB/s, avg write rate: 114.609 MB/s system usage: CPU: user: 0.00 s, system: 0.00 s, elapsed: 0.00 s',
            "PID": 22113,
            "SEVERITY": "notice",
            "_pgsql": {
                "application_name": "",
                "command_tag": "",
                "connection_from": "localhost",
                "context": "",
                "database": "",
                "detail": "",
                "hint": "",
                "internal_query": "",
                "location": "",
                "message": r'automatic vacuum of table "tablename": index scans: 0 pages: 0 removed, 4 remain, 0 skipped due to pins, 0 skipped frozen tuples: 114 removed, 268 remain, 0 are dead but not yet removable, oldest xmin: 149738000 buffer usage: 97 hits, 0 misses, 6 dirtied avg read rate: 0.000 MB/s, avg write rate: 114.609 MB/s system usage: CPU: user: 0.00 s, system: 0.00 s, elapsed: 0.00 s',
                "pid": 22113,
                "query": "",
                "session_id": "64d22fa0.5661",
                "session_line_num": 1,
                "session_start_time": "2023-08-08 12:05:52 UTC",
                "severity": "WARNING",
                "sql_state_code": "00000",
                "timestamp": "2023-08-08 12:05:52.805 UTC",
                "transaction_id": "0",
                "username": "",
                "virtual_transaction_id": "",
            },
        },
    ),
    (
        r'2023-08-08 12:05:52.805 UTC,,,22113,,64d22fa0.5661,1,,2023-08-08 12:05:52 UTC,23/74060,0,PANIC,00000,"automatic vacuum of table ""tablename"": index scans: 0 pages: 0 removed, 4 remain, 0 skipped due to pins, 0 skipped frozen tuples: 114 removed, 268 remain, 0 are dead but not yet removable, oldest xmin: 149738000 buffer usage: 97 hits, 0 misses, 6 dirtied avg read rate: 0.000 MB/s, avg write rate: 114.609 MB/s system usage: CPU: user: 0.00 s, system: 0.00 s, elapsed: 0.00 s",,,,,,,,,""',
        {
            "DATE": "Aug  8 12:05:52",
            "HOST_FROM": "localhost",
            "MSG": r'automatic vacuum of table "tablename": index scans: 0 pages: 0 removed, 4 remain, 0 skipped due to pins, 0 skipped frozen tuples: 114 removed, 268 remain, 0 are dead but not yet removable, oldest xmin: 149738000 buffer usage: 97 hits, 0 misses, 6 dirtied avg read rate: 0.000 MB/s, avg write rate: 114.609 MB/s system usage: CPU: user: 0.00 s, system: 0.00 s, elapsed: 0.00 s',
            "PID": 22113,
            "SEVERITY": "crit",
            "_pgsql": {
                "application_name": "",
                "command_tag": "",
                "connection_from": "localhost",
                "context": "",
                "database": "",
                "detail": "",
                "hint": "",
                "internal_query": "",
                "location": "",
                "message": r'automatic vacuum of table "tablename": index scans: 0 pages: 0 removed, 4 remain, 0 skipped due to pins, 0 skipped frozen tuples: 114 removed, 268 remain, 0 are dead but not yet removable, oldest xmin: 149738000 buffer usage: 97 hits, 0 misses, 6 dirtied avg read rate: 0.000 MB/s, avg write rate: 114.609 MB/s system usage: CPU: user: 0.00 s, system: 0.00 s, elapsed: 0.00 s',
                "pid": 22113,
                "query": "",
                "session_id": "64d22fa0.5661",
                "session_line_num": 1,
                "session_start_time": "2023-08-08 12:05:52 UTC",
                "severity": "WARNING",
                "sql_state_code": "00000",
                "timestamp": "2023-08-08 12:05:52.805 UTC",
                "transaction_id": "0",
                "username": "",
                "virtual_transaction_id": "",
            },
        },
    ),
    (
        # Less fields than specified in the scl
        r'2023-08-08 12:05:52.805 UTC,,,22113,,64d22fa0.5661,1,,2023-08-08 12:05:52 UTC,23/74060,0,LOG,00000,',
        {

            "DATE": "Aug  8 12:05:52",
            "HOST_FROM": "localhost",
            "PID": 22113,
            "SEVERITY": "info",
            "_pgsql": {
                "application_name": "",
                "command_tag": "",
                "connection_from": "localhost",
                "context": "",
                "database": "",
                "detail": "",
                "hint": "",
                "internal_query": "",
                "location": "",
                "pid": 22113,
                "query": "",
                "session_id": "64d22fa0.5661",
                "session_line_num": 1,
                "session_start_time": "2023-08-08 12:05:52 UTC",
                "severity": "LOG",
                "sql_state_code": "00000",
            },
        },
    ),
    (
        # Log with query_id(v14+), backend_type(v13+)
        r'2023-11-03 16:32:35.084 CET,"postgres","bench_test",3451998,"[local]",65451258.34ac5e,632376,"UPDATE",2023-11-03 16:31:36 CET,6/45171,228887,LOG,00000,"duration: 0.081 ms",,,,,,,,,"pgbench","client backend",,1521477082073268809',
        {
            "DATE": "Nov  3 16:32:35",
            "HOST_FROM": "localhost",
            "MSG": r'automatic vacuum of table "tablename": index scans: 0 pages: 0 removed, 4 remain, 0 skipped due to pins, 0 skipped frozen tuples: 114 removed, 268 remain, 0 are dead but not yet removable, oldest xmin: 149738000 buffer usage: 97 hits, 0 misses, 6 dirtied avg read rate: 0.000 MB/s, avg write rate: 114.609 MB/s system usage: CPU: user: 0.00 s, system: 0.00 s, elapsed: 0.00 s',
            "PID": 3451998,
            "SEVERITY": "info",
            "_pgsql": {
                "application_name": "",
                "command_tag": "",
                "connection_from": "localhost",
                "context": "",
                "database": "",
                "detail": "",
                "hint": "",
                "internal_query": "",
                "location": "",
                "message": r'automatic vacuum of table "tablename": index scans: 0 pages: 0 removed, 4 remain, 0 skipped due to pins, 0 skipped frozen tuples: 114 removed, 268 remain, 0 are dead but not yet removable, oldest xmin: 149738000 buffer usage: 97 hits, 0 misses, 6 dirtied avg read rate: 0.000 MB/s, avg write rate: 114.609 MB/s system usage: CPU: user: 0.00 s, system: 0.00 s, elapsed: 0.00 s',
                "pid": 22113,
                "query": "",
                "session_id": "65451258.34ac5e",
                "session_line_num": 632376,
                "session_start_time": "2023-11-03 16:31:36 CET",
                "severity": "LOG",
                "sql_state_code": "00000",
                "timestamp": "2023-11-03 16:32:35.084 CET",
                "transaction_id": "0",
                "username": "",
                "virtual_transaction_id": "",
                "query_id": 1521477082073268809,
                "backend_type": "client backend",
            },
        },
    ),
]


@pytest.mark.parametrize(
    "input_message, expected_kv_pairs",
    test_parameters,
    ids=range(len(test_parameters)),
)
def test_postgresql_csvlog_parser(config, port_allocator, syslog_ng, input_message, expected_kv_pairs):
    config.add_include("scl.conf")

    network_source = config.create_network_source(ip="localhost", port=port_allocator(), flags="no-parse")
    pgsql_csvlog_parser = config.create_postgresql_csvlog_parser()
    file_destination = config.create_file_destination(
        file_name="output.log",
        template=config.stringify("$(format-json --scope everything)" + "\n"),
    )
    config.create_logpath(
        statements=[network_source, pgsql_csvlog_parser, file_destination],
    )

    syslog_ng.start(config)
    network_source.write_log(input_message)

    output_json = json.loads(file_destination.read_log())

    assert output_json["DATE"] == expected_kv_pairs["DATE"]
    assert output_json["HOST_FROM"] == expected_kv_pairs["HOST_FROM"]
    assert output_json["PID"] == expected_kv_pairs["PID"]
    assert output_json["SEVERITY"] == expected_kv_pairs["SEVERITY"]

    assert output_json["_pgsql"]["session_id"] == expected_kv_pairs["_pgsql"]["session_id"]
    assert output_json["_pgsql"]["session_line_num"] == expected_kv_pairs["_pgsql"]["session_line_num"]
    assert output_json["_pgsql"]["session_start_time"] == expected_kv_pairs["_pgsql"]["session_start_time"]

    # If field is filled
    if "timestamp" in expected_kv_pairs["_pgsql"]:
        assert output_json["_pgsql"]["timestamp"] == expected_kv_pairs["_pgsql"]["timestamp"]

    # Optional, extra fields
    extra_fields = ["query_id", "backend_type"]
    for extra_field in extra_fields:
        if extra_field in expected_kv_pairs["_pgsql"]:
            assert output_json["_pgsql"][extra_field] == expected_kv_pairs["_pgsql"][extra_field]



#############################################################################
# Copyright (c) 2015-2016 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

from __future__ import print_function, absolute_import
from .test_completer import CompleterTestCase
from ..langcompleter import LangBasedCompleter
from ..completerlang import CompleterLang
from ..choicecompleter import ChoiceCompleter
from ..lexertoken import LexerToken


class DummyLang(CompleterLang):
    # pylint: disable=super-init-not-called
    def __init__(self, expected_tokens, replaced_token=None, replaced_token_pos=-1):
        self._expected_tokens = expected_tokens
        self._replaced_token = replaced_token
        self._replaced_token_pos = replaced_token_pos

    def _construct_lexer(self):
        pass

    def get_expected_tokens(self, text, drop_last_token):
        if self._replaced_token is not None:
            replaced_token = LexerToken(type=self._expected_tokens,
                                        value=self._replaced_token,
                                        lexpos=self._replaced_token_pos)
        else:
            replaced_token = None
        return (self._expected_tokens, replaced_token, self._replaced_token_pos)


class TestLangCompleter(CompleterTestCase):
    default_expectation = "1ST_TOKEN"
    default_completers = {
        '1ST_TOKEN': ChoiceCompleter(("token1-a", "token1-b"), suffix=''),
        '2ND_TOKEN': ChoiceCompleter(("token2-a", "token2-b"), suffix=''),
        '3RD_TOKEN': ChoiceCompleter(("token3-a", "token3-b"), suffix=''),
        'PARTIAL_TOKEN': ChoiceCompleter(("tokenP-a", "tokenP-b"), prefix='@', suffix='')
    }

    # pylint: disable=arguments-differ,too-many-arguments,too-many-positional-arguments
    def _construct_completer(self, expected_token=None, expected_tokens=None,
                             replaced_token=None, replaced_token_pos=-1,
                             completers=None, prefix="<!--"):
        if expected_tokens is None:
            expected_tokens = [expected_token or self.default_expectation]
        return LangBasedCompleter(parser=DummyLang(expected_tokens=expected_tokens,
                                                   replaced_token=replaced_token,
                                                   replaced_token_pos=replaced_token_pos),
                                  completers=completers or self.default_completers,
                                  prefix=prefix)

    def test_completing_on_the_first_characters_of_prefix_offers_the_prefix(self):
        self._assert_completions_offered("", expected_completions=["<!--"])
        self._assert_completions_offered("<", expected_completions=["<!--"])
        self._assert_completions_offered("<!", expected_completions=["<!--"])
        self._assert_completions_offered("<!-", expected_completions=["<!--"])

    def test_completing_on_an_input_with_a_mismatching_prefix(self):
        self._assert_no_completions_are_offered("mismatch")

    def test_completing_on_prefix_only_offers_the_completions_of_the_first_token_with_prefix_prepended(self):
        self._assert_completions_offered("<!--", expected_completions=["<!--token1-a", "<!--token1-b"])

    def test_completing_the_1st_empty_token_offers_completions_on_the_token(self):
        self._completer = self._construct_completer(expected_token="1ST_TOKEN")
        self._assert_completions_offered(entire_input="<!-- ", word="", expected_completions=["token1-a", "token1-b"])

    def test_completing_the_2nd_empty_token_offers_completions_on_the_token(self):
        self._completer = self._construct_completer(expected_token="2ND_TOKEN")
        self._assert_completions_offered(entire_input="<!-- foo ", word="",
                                         expected_completions=['token2-a', 'token2-b'])

    def test_completing_the_3rd_empty_token_offers_completions_on_the_token(self):
        self._completer = self._construct_completer(expected_token="3RD_TOKEN")
        self._assert_completions_offered(entire_input="<!-- foo bar ", word="",
                                         expected_completions=['token3-a', 'token3-b'])

    def test_completing_a_partial_token_with_an_empty_word_produces_choices_with_the_partial_prefix(self):
        self._completer = self._construct_completer(expected_token="PARTIAL_TOKEN")
        self._assert_completions_offered(entire_input="<!-- foo ", word="",
                                         expected_completions=['@'])

    def test_completing_a_partial_token_with_only_the_prefix_produces_choices_with_the_partial_prefix(self):
        # NOTE: replaced_token_pos is calculated after the language prefix (<!--) is chopped!
        self._completer = self._construct_completer(expected_token="PARTIAL_TOKEN",
                                                    replaced_token="@",
                                                    replaced_token_pos=5)
        self._assert_completions_offered(entire_input="<!-- foo @", word="@",
                                         expected_completions=['@tokenP-a', '@tokenP-b'])

    def test_completing_a_partial_token_with_a_few_matching_characters_produces_choices_with_the_partial_prefix(self):
        # NOTE: replaced_token_pos is calculated after the language prefix (<!--) is chopped!
        self._completer = self._construct_completer(expected_token="PARTIAL_TOKEN",
                                                    replaced_token="@tok",
                                                    replaced_token_pos=5)
        self._assert_completions_offered(entire_input="<!-- foo @tok", word="@tok",
                                         expected_completions=['@tokenP-a', '@tokenP-b'])

    def test_completing_a_partial_token_with_unmatching_characters_produces_no_completions(self):
        # NOTE: replaced_token_pos is calculated after the language prefix (<!--) is chopped!
        self._completer = self._construct_completer(expected_token="PARTIAL_TOKEN",
                                                    replaced_token="@unmatching",
                                                    replaced_token_pos=5)
        self._assert_no_completions_are_offered(entire_input="<!-- foo @unmatching", word="@unmatching")

    def test_completing_a_partial_token_that_is_longer_than_a_word_produces_no_matches(self):

        # This is pretty much a corner case, as this would mean that the word
        # splitting features of readline is not in-line with the tokenization
        # rules of the language. A sample is encoded below in the testcase,
        # the word itself is shorter than the token, which means that we
        # basically can't complete on the entire token as readline would only replace
        # stuff partially.
        #
        # The best we can do here is to offer no completions, the syntax we wanted
        # to extend didn't have such a case, fortunately.

        # NOTE: replaced_token_pos is calculated after the language prefix (<!--) is chopped!
        self._completer = self._construct_completer(expected_token="PARTIAL_TOKEN",
                                                    replaced_token="@token tail",
                                                    replaced_token_pos=5)
        self._assert_no_completions_are_offered(entire_input="<!-- foo @token tail", word="tail")

    def test_completing_a_token_where_multiple_tokens_could_match_collects_all_matches(self):
        self._completer = self._construct_completer(expected_tokens=["1ST_TOKEN", "2ND_TOKEN", "3RD_TOKEN"])
        self._assert_completions_offered(entire_input="<!-- ", word="",
                                         expected_completions=["token1-a", "token2-b", "token3-a"])
        self._assert_completions_not_offered(entire_input="<!-- ", word="",
                                             unexpected_completions=["@tokenP-a"])

    def test_completing_a_token_that_has_no_registered_completer_results_in_no_matches(self):
        self._completer = self._construct_completer(expected_token="NOSUCHCOMPLETER")
        self._assert_no_completions_are_offered("<!-- ")



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2020 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from src.syslog_ng_config.statements.filters.filter import Match


def test_set_tag(config, syslog_ng):
    match = Match(config.stringify("MATCHSTRING"), value=config.stringify("MSG"))
    notmatch = Match(config.stringify("NONE"), value=config.stringify("MSG"))

    generator_source = config.create_example_msg_generator_source(num=1, template=config.stringify("input with MATCHSTRING in it"))
    set_tag_with_matching = config.create_rewrite_set_tag(config.stringify("SHOULDMATCH"), condition=match)
    set_tag_without_matching = config.create_rewrite_set_tag(config.stringify("DONOTMATCH"), condition=notmatch)
    file_destination = config.create_file_destination(file_name="output.log", template=config.stringify('${TAGS}\n'))
    config.create_logpath(statements=[generator_source, set_tag_with_matching, set_tag_without_matching, file_destination])

    syslog_ng.start(config)
    log_line = file_destination.read_log().strip()
    assert "SHOULDMATCH" in log_line
    assert "DONOTMATCH" not in log_line


def test_set_tag_with_template(config, syslog_ng):
    generator_source = config.create_example_msg_generator_source(num=1, template=config.stringify("FOO"))
    set_tag_with_template = config.create_rewrite_set_tag(config.stringify("TAG-${MSG}"))
    file_destination = config.create_file_destination(file_name="output.log", template=config.stringify('${TAGS}\n'))
    config.create_logpath(statements=[generator_source, set_tag_with_template, file_destination])

    syslog_ng.start(config)
    log_line = file_destination.read_log().strip()
    assert "TAG-FOO" in log_line



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2023 Balázs Scheidler <balazs.scheidler@axoflow.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################


def test_template_stmt_with_identifier_reference(config, syslog_ng):
    template = config.create_template("template with $(format-welf test.*)\n")
    config.add_template(template)

    generator_source = config.create_example_msg_generator_source(num=1, values="test.key1 => value1 test.key2 => value2")
    file_destination = config.create_file_destination(file_name="output.log", template=template.name)

    config.create_logpath(statements=[generator_source, file_destination])
    syslog_ng.start(config)
    log = file_destination.read_logs(1)
    assert log == ["template with test.key1=value1 test.key2=value2\n"]


def test_template_with_non_string_values(config, syslog_ng):

    generator_source = config.create_example_msg_generator_source(num=1, values="test.key1 => value1 test.key2 => value2")
    rewrites = [
        config.create_rewrite_set("int(10)", value="values_int_hint"),
        config.create_rewrite_set("float(4.5)", value="values_float_hint"),
        config.create_rewrite_set("10", value="values_int_literal"),
        config.create_rewrite_set("4.5", value="values_float_literal"),
    ]
    file_destination = config.create_file_destination(file_name="output.log", template=config.stringify("$(format-json values_*)\n"))

    config.create_logpath(statements=[generator_source, *rewrites, file_destination])
    syslog_ng.start(config)
    log = file_destination.read_logs(1)
    assert log == ["""{"values_int_literal":10,"values_int_hint":10,"values_float_literal":4.5,"values_float_hint":4.5}\n"""]


def test_template_stmt_with_indirect_invocation(config, syslog_ng):
    template = config.create_template("template with $(format-welf test.*)\n")
    config.add_template(template)

    generator_source = config.create_example_msg_generator_source(num=1, values="test.key1 => value1 test.key2 => value2 template_fn => {}".format(template.name))
    file_destination = config.create_file_destination(file_name="output.log", template=config.stringify("$(template {} error resolving template)\n".format(template.name)))

    config.create_logpath(statements=[generator_source, file_destination])
    syslog_ng.start(config)
    log = file_destination.read_logs(1)
    assert log == ["template with test.key1=value1 test.key2=value2\n"]


def test_simple_template_stmt_with_identifier_reference(config, syslog_ng):
    template = config.create_template("template with $(format-welf test.*)\n", use_simple_statement=True)
    config.add_template(template)

    generator_source = config.create_example_msg_generator_source(num=1, values="test.key1 => value1 test.key2 => value2")
    file_destination = config.create_file_destination(file_name="output.log", template=template.name)

    config.create_logpath(statements=[generator_source, file_destination])
    syslog_ng.start(config)
    log = file_destination.read_logs(1)
    assert log == ["template with test.key1=value1 test.key2=value2\n"]


def test_template_stmt_with_string_reference(config, syslog_ng):
    template = config.create_template("template with $(format-welf test.*)\n")
    config.add_template(template)

    generator_source = config.create_example_msg_generator_source(num=1, values="test.key1 => value1 test.key2 => value2")
    file_destination = config.create_file_destination(file_name="output.log", template=config.stringify(template.name))

    config.create_logpath(statements=[generator_source, file_destination])
    syslog_ng.start(config)
    log = file_destination.read_logs(1)
    assert log == ["template with test.key1=value1 test.key2=value2\n"]


def test_inline_template(config, syslog_ng):
    generator_source = config.create_example_msg_generator_source(num=1, values="test.key1 => value1 test.key2 => value2")
    file_destination = config.create_file_destination(file_name="output.log", template=config.stringify("template with $(format-welf test.*)\n"))

    config.create_logpath(statements=[generator_source, file_destination])
    syslog_ng.start(config)
    log = file_destination.read_logs(1)
    assert log == ["template with test.key1=value1 test.key2=value2\n"]


def test_inline_template_hints(config, syslog_ng):
    generator_source = config.create_example_msg_generator_source(num=1, values="test.key1 => int(5) test.key2 => int(6)")
    file_destination = config.create_file_destination(file_name="output.log", template=config.stringify("template with $(format-json test.*)\n"))

    config.create_logpath(statements=[generator_source, file_destination])
    syslog_ng.start(config)
    log = file_destination.read_logs(1)
    # NOTE: both key1 and key2 are formatted as integers
    assert log == ['''template with {"test":{"key2":6,"key1":5}}\n''']


def test_value_pair_type_hints(config, syslog_ng):
    generator_source = config.create_example_msg_generator_source(num=1, values="test.key1 => string(5) test.key2 => string(6)")
    file_destination = config.create_file_destination(file_name="output.log", template=config.stringify("template with $(format-json v1=int(${test.key1}))\n"))

    config.create_logpath(statements=[generator_source, file_destination])
    syslog_ng.start(config)
    log = file_destination.read_logs(1)
    # NOTE: v1 is formatted as an integer due to the type hint in the format-json option list
    assert log == ['''template with {"v1":5}\n''']


def test_template_function(config, syslog_ng):
    template = config.create_template_function("template with $(format-welf test.*)\n", name="test_template_fn")
    config.add_template(template)

    generator_source = config.create_example_msg_generator_source(num=1, values="test.key1 => value1 test.key2 => value2")
    file_destination = config.create_file_destination(file_name="output.log", template=config.stringify("$(test_template_fn)\n"))

    config.create_logpath(statements=[generator_source, file_destination])
    syslog_ng.start(config)
    log = file_destination.read_logs(1)
    assert log == ["template with test.key1=value1 test.key2=value2\n"]



#############################################################################
# Copyright (c) 2015-2016 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

from __future__ import absolute_import, print_function
from ..debuglang import DebugLang
from .test_completerlang import CompleterLangTestCase


class TestDebugLang(CompleterLangTestCase):
    def setUp(self):
        self._parser = DebugLang()

    def test_first_token_is_a_command(self):
        self._assert_command_token_follows("")
        self._assert_command_token_follows(" ")
        self._assert_command_token_follows("foo")

    def test_print_expects_a_template_arg(self):
        self._assert_token_follows("print ", ["template", "ARG"])
        self._assert_token_follows("print $MSG ", [])

    def test_display_expects_a_template_arg(self):
        self._assert_token_follows("display ", ["template", "ARG"])
        self._assert_token_follows("display $MSG ", [])

    def test_generic_command_expects_a_series_of_args(self):
        self._assert_token_follows("foo ", ["ARG"])
        self._assert_token_follows("foo arg ", ["ARG"])
        self._assert_token_follows("foo arg arg ", ["ARG"])

    def _get_command_tokens(self):
        return [token for token in self._parser.tokens if token.startswith("COMMAND")]

    def _assert_command_token_follows(self, text, pos=None, replaced_token=None):
        self._assert_token_follows(text, set(self._get_command_tokens()), pos, replaced_token)



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2021 One Identity
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################


def test_no_header_flag(config, syslog_ng, log_message, bsd_formatter):
    input_message = bsd_formatter.format_message(log_message)

    file_source = config.create_file_source(file_name="input.log", flags="no-header")
    file_destination = config.create_file_destination(file_name="output.log", template="'<$PRI>$MSG\n'")
    config.create_logpath(statements=[file_source, file_destination])

    file_source.write_log(input_message)
    syslog_ng.start(config)

    expected_msg_value_for_no_header_flag = "%s %s %s[%s]: %s\n" % (
        log_message.bsd_timestamp_value,
        log_message.hostname_value,
        log_message.program_value,
        log_message.pid_value,
        log_message.message_value,
    )
    full_expected_message = "<{}>{}".format(log_message.priority_value, expected_msg_value_for_no_header_flag)

    assert file_destination.read_log() == full_expected_message



#############################################################################
# Copyright (c) 2015-2016 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

from __future__ import print_function, absolute_import
import unittest


class CompleterTestCase(unittest.TestCase):
    def setUp(self):
        self._completer = self._construct_completer()

    def _construct_completer(self):
        raise NotImplementedError

    def _get_completions(self, word, entire_input=None):
        return self._completer.complete(entire_input or word, word)

    def _assert_completion_offered(self, word, completion):
        self._assert_completions_offered(word, [completion])

    def _assert_completion_not_offered(self, word, completion):
        self._assert_completions_not_offered(word, [completion])

    def _assert_no_completions_are_offered(self, word, entire_input=None):
        completions = self._get_completions(word, entire_input=entire_input)
        self.assertEqual(completions, [])

    def _assert_completions_offered(self, word, expected_completions, entire_input=None):
        completions = self._get_completions(word, entire_input=entire_input)
        for completion in expected_completions:
            self.assertIn(completion, completions)
        self.assertEqual(completions, sorted(completions))

    def _assert_completions_not_offered(self, word, unexpected_completions, entire_input=None):
        completions = self._get_completions(word, entire_input=entire_input)
        for completion in unexpected_completions:
            self.assertNotIn(completion, completions)

    def _assert_completions_start_with_word(self, word):
        completions = self._get_completions(word)
        for completion in completions:
            self.assertTrue(completion.startswith(word),
                            msg=f"Completion {completion} does not start with the word to be completed {word}")



#############################################################################
# Copyright (c) 2015-2016 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

from __future__ import print_function, absolute_import
from .test_completer import CompleterTestCase
from ..choicecompleter import ChoiceCompleter


class TestChoiceCompleter(CompleterTestCase):
    # pylint: disable=arguments-differ
    def _construct_completer(self, choices=None, prefix=None, suffix=None):
        return ChoiceCompleter(choices or ['foo', 'bar', 'baz'],
                               prefix=prefix,
                               suffix=suffix)

    def test_all_choices_are_offered_for_an_empty_string(self):
        self._completer = self._construct_completer(suffix='')
        self._assert_completions_offered('', ['foo', 'bar', 'baz'])

    def test_only_completions_that_start_with_word_are_listed_as_completions(self):
        for word in ('f', 'b'):
            self._assert_completions_start_with_word(word)

    def test_suffix_is_attached_to_all_potential_matches(self):
        self._completer = self._construct_completer(suffix=' ')
        self._assert_completions_offered('b', ['bar ', 'baz '])
        self._assert_completions_not_offered('b', ['bar', 'baz'])

    def test_prefix_is_attached_to_all_potential_matches(self):
        self._completer = self._construct_completer(prefix='$', suffix='')
        self._assert_completions_offered('$b', ['$bar', '$baz'])
        self._assert_completions_not_offered('$b', ['bar', 'baz'])

    def test_only_prefix_is_offered_if_entire_input_is_shorter(self):
        self._completer = self._construct_completer(prefix='$@', suffix='@')
        self._assert_completion_offered('', '$@')
        self._assert_completion_offered('$', '$@')
        self._assert_completion_not_offered('', '$@bar@')
        self._assert_completion_not_offered('$', '$@bar@')
        self._assert_completions_offered('$@', ['$@foo@', '$@bar@', '$@baz@'])

    def test_not_even_the_prefix_is_offered_if_the_entire_input_diverged_already(self):
        self._completer = self._construct_completer(prefix='$@', suffix='@')
        self._assert_completions_not_offered('q', ['$', '$@', '$@foo@'])



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2018 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from enum import Enum  # noreorder

from pathlib import Path

from src.executors.command_executor import CommandExecutor


class QueryTypes(Enum):
    QUERY_GET = 0
    QUERY_SUM = 1
    QUERY_LIST = 2
    QUERY_GET_RESET = 3


class SyslogNgCtlExecutor(object):
    def __init__(self, instance_paths):
        self.__instance_paths = instance_paths
        self.__syslog_ng_control_tool_path = instance_paths.get_syslog_ng_ctl_bin()
        self.__syslog_ng_control_socket_path = instance_paths.get_control_socket_path()
        self.__command_executor = CommandExecutor()

    def run_command(self, command_short_name, command):
        return self.__command_executor.run(
            command=self.__construct_ctl_command(command),
            stdout_path=self.__construct_std_file_path(command_short_name, "stdout"),
            stderr_path=self.__construct_std_file_path(command_short_name, "stderr"),
        )

    def __construct_std_file_path(self, command_short_name, std_type):
        instance_name = self.__instance_paths.get_instance_name()
        return Path("syslog_ng_ctl_{}_{}_{}".format(instance_name, command_short_name, std_type))

    def __construct_ctl_command(self, command):
        ctl_command = [self.__syslog_ng_control_tool_path]
        ctl_command += command
        ctl_command.append("--control={}".format(self.__syslog_ng_control_socket_path))
        return ctl_command

    @staticmethod
    def construct_ctl_stats_command(reset):
        stats_command = ["stats"]
        if reset:
            stats_command.append("--reset")
        return stats_command

    @staticmethod
    def construct_ctl_stats_prometheus_command():
        return ["stats", "prometheus"]

    @staticmethod
    def construct_ctl_credentials_command(credential, secret):
        return ["credentials", "add", credential, secret]

    @staticmethod
    def construct_ctl_query_command(pattern, query_type):
        query_command = ["query"]

        if query_type == QueryTypes.QUERY_GET:
            query_command += ["get"]
        elif query_type == QueryTypes.QUERY_SUM:
            query_command += ["get --sum"]
        elif query_type == QueryTypes.QUERY_LIST:
            query_command += ["list"]
        elif query_type == QueryTypes.QUERY_GET_RESET:
            query_command += ["get --reset"]

        query_command += [pattern]

        return query_command



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2020 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import pytest

test_parameters_raw = [
    # Testing SYSTEM branch
    (r"""<12>Apr 14 16:48:54 paloalto.test.net 1,2020/04/14 16:48:54,unknown,SYSTEM,auth,0,2020/04/14 16:48:54,,auth-fail,,0,0,general,medium,failed authentication for user 'admin'. Reason: Invalid username/password. From: 10.0.10.55.,1718,0x0,0,0,0,0,,paloalto""", "<${PROGRAM}><${.panos.type}><${.panos.eventid}><${.panos.device_name}>", "<paloalto_panos><SYSTEM><auth-fail><paloalto>"),
    # Testing SYSTEM branch with extra additions at the end. Should be accepted.
    (r"""<12>Apr 14 16:48:54 paloalto.test.net 1,2020/04/14 16:48:54,unknown,SYSTEM,auth,0,2020/04/14 16:48:54,,auth-fail,,0,0,general,medium,failed authentication for user 'admin'. Reason: Invalid username/password. From: 10.0.10.55.,1718,0x0,0,0,0,0,,paloalto,foo,bar""", "<${PROGRAM}><${.panos.type}><${.panos.eventid}><${.panos.device_name}>", "<paloalto_panos><SYSTEM><auth-fail><paloalto>"),
    # Testing CONFIG branch
    (r"""<14>Apr 14 16:54:18 paloalto.test.net 1,2020/04/14 16:54:18,unknown,CONFIG,0,0,2020/04/14 16:54:18,10.0.10.55,,set,admin,Web,Succeeded,deviceconfig system,127,0x0,0,0,0,0,,paloalto""", "<${PROGRAM}><${.panos.type}><${.panos.path}><${.panos.device_name}>", "<paloalto_panos><CONFIG><deviceconfig system><paloalto>"),
    # Testing CONFIG branch, custom logs
    (r"""<14>Apr 14 16:54:18 paloalto.test.net 1,2020/04/14 16:54:18,unknown,CONFIG,0,0,2020/04/14 16:54:18,10.0.10.55,,set,admin,Web,Succeeded,deviceconfig system,before,after,127,0x0,0,0,0,0,,paloalto""", "<${PROGRAM}><${.panos.type}><${.panos.path}><${.panos.device_name}>", "<paloalto_panos><CONFIG><deviceconfig system><paloalto>"),
]


@pytest.mark.parametrize(
    "input_message, template, expected_value", test_parameters_raw,
    ids=list(map(str, range(len(test_parameters_raw)))),
)
def test_panos_parser(config, syslog_ng, input_message, template, expected_value):
    config.add_include("scl.conf")

    generator_source = config.create_example_msg_generator_source(num=1, template=config.stringify(input_message))
    checkpoint_parser = config.create_panos_parser()

    file_destination = config.create_file_destination(file_name="output.log", template=config.stringify(template + "\n"))
    config.create_logpath(statements=[generator_source, checkpoint_parser, file_destination])

    syslog_ng.start(config)
    assert file_destination.read_log().strip() == expected_value



#############################################################################
# Copyright (c) 2022 Balazs Scheidler <bazsi77@gmail.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from syslogng.modules.kubernetes import KubernetesAPIEnrichment
from syslogng.message import LogMessage
from syslogng.template import LogTemplate
import pytest
import json

import kubernetes.client


@pytest.fixture
def minimal_config():
    return {
        'prefix': 'myk8s.',
    }


@pytest.fixture
def mock_kube_config(mocker):
    client_configuration = kubernetes.client.Configuration()
    client_configuration.host = "https://dummy.kube.cluster:9090/"
    client_configuration.api_key = "bearer dummytoken"
    kubernetes.client.Configuration.set_default(client_configuration)

    yield mocker.patch('kubernetes.config.load_config', return_value=None)

    kubernetes.client.Configuration.set_default(None)


@pytest.fixture
def mock_api_response_nginx(mocker):
    response_json = {
        'kind': 'PodList',
        'apiVersion': 'v1',
        'metadata': {'selfLink': '/api/v1/namespaces/default/pods', 'resourceVersion': '738405'},
        'items': [
            {
                'metadata': {
                    'name': 'nginx-85b98978db-2dpdh',
                    'generateName': 'nginx-85b98978db-',
                    'namespace': 'default',
                    'selfLink': '/api/v1/namespaces/default/pods/nginx-85b98978db-2dpdh',
                    'uid': 'd7a782cf-cfaa-45e6-8f22-950b2df5bf82',
                    'resourceVersion': '738230',
                    'creationTimestamp': '2022-04-20T08:08:37Z',
                    'labels': {'app': 'nginx', 'pod-template-hash': '85b98978db'},
                    'annotations': {
                        'cni.projectcalico.org/podIP': '10.1.71.91/32',
                        'cni.projectcalico.org/podIPs': '10.1.71.91/32'
                    },
                    'ownerReferences': [
                        {
                            'apiVersion': 'apps/v1',
                            'kind': 'ReplicaSet',
                            'name': 'nginx-85b98978db',
                            'uid': '2958d7c9-f6c0-4f65-8943-0c6dd54bb98f',
                            'controller': True,
                            'blockOwnerDeletion': True
                        }
                    ],
                    'managedFields': [
                        {
                            'manager': 'kubelite',
                            'operation': 'Update',
                            'apiVersion': 'v1',
                            'time': '2022-04-20T08:08:37Z',
                            'fieldsType': 'FieldsV1',
                            'fieldsV1': {'f:metadata': {'f:generateName': {}, 'f:labels': {'.': {}, 'f:app': {}, 'f:pod-template-hash': {}}, 'f:ownerReferences': {'.': {}, 'k:{"uid":"2958d7c9-f6c0-4f65-8943-0c6dd54bb98f"}': {}}}, 'f:spec': {'f:containers': {'k:{"name":"nginx"}': {'.': {}, 'f:image': {}, 'f:imagePullPolicy': {}, 'f:name': {}, 'f:resources': {}, 'f:terminationMessagePath': {}, 'f:terminationMessagePolicy': {}}}, 'f:dnsPolicy': {}, 'f:enableServiceLinks': {}, 'f:restartPolicy': {}, 'f:schedulerName': {}, 'f:securityContext': {}, 'f:terminationGracePeriodSeconds': {}}}
                        },
                        {
                            'manager': 'calico',
                            'operation': 'Update',
                            'apiVersion': 'v1',
                            'time': '2022-04-20T08:08:38Z',
                            'fieldsType': 'FieldsV1',
                            'fieldsV1': {'f:metadata': {'f:annotations': {'.': {}, 'f:cni.projectcalico.org/podIP': {}, 'f:cni.projectcalico.org/podIPs': {}}}},
                            'subresource': 'status'
                        },
                        {
                            'manager': 'Go-http-client',
                            'operation': 'Update',
                            'apiVersion': 'v1',
                            'time': '2022-04-29T19:56:29Z',
                            'fieldsType': 'FieldsV1',
                            'fieldsV1': {'f:status': {'f:conditions': {'k:{"type":"ContainersReady"}': {'.': {}, 'f:lastProbeTime': {}, 'f:type': {}}, 'k:{"type":"Initialized"}': {'.': {}, 'f:lastProbeTime': {}, 'f:lastTransitionTime': {}, 'f:status': {}, 'f:type': {}}, 'k:{"type":"Ready"}': {'.': {}, 'f:lastProbeTime': {}, 'f:type': {}}}, 'f:phase': {}, 'f:startTime': {}}},
                            'subresource': 'status'
                        },
                        {
                            'manager': 'kubelite',
                            'operation': 'Update',
                            'apiVersion': 'v1',
                            'time': '2022-11-01T12:04:11Z',
                            'fieldsType': 'FieldsV1',
                            'fieldsV1': {'f:status': {'f:conditions': {'k:{"type":"ContainersReady"}': {'f:lastTransitionTime': {}, 'f:status': {}}, 'k:{"type":"Ready"}': {'f:lastTransitionTime': {}, 'f:status': {}}}, 'f:containerStatuses': {}, 'f:hostIP': {}, 'f:podIP': {}, 'f:podIPs': {'.': {}, 'k:{"ip":"10.1.71.91"}': {'.': {}, 'f:ip': {}}}}},
                            'subresource': 'status'
                        }
                    ]
                },
                'spec': {
                    'volumes': [
                        {
                            'name': 'kube-api-access-7xzch',
                            'projected': {
                                'sources': [
                                    {
                                        'serviceAccountToken': {
                                            'expirationSeconds': 3607,
                                            'path': 'token'}
                                    },
                                    {
                                        'configMap': {
                                            'name': 'kube-root-ca.crt',
                                            'items': [{'key': 'ca.crt', 'path': 'ca.crt'}]
                                        }
                                    },
                                    {
                                        'downwardAPI': {
                                            'items': [{'path': 'namespace', 'fieldRef': {'apiVersion': 'v1', 'fieldPath': 'metadata.namespace'}}]
                                        }
                                    }
                                ],
                                'defaultMode': 420}
                        }
                    ],
                    'containers': [
                        {
                            'name': 'nginx',
                            'image': 'nginx',
                            'resources': {},
                            'volumeMounts': [{'name': 'kube-api-access-7xzch', 'readOnly': True, 'mountPath': '/var/run/secrets/kubernetes.io/serviceaccount'}],
                            'terminationMessagePath': '/dev/termination-log',
                            'terminationMessagePolicy': 'File',
                            'imagePullPolicy': 'Always'
                        }
                    ],
                    'restartPolicy': 'Always',
                    'terminationGracePeriodSeconds': 30,
                    'dnsPolicy': 'ClusterFirst',
                    'serviceAccountName': 'default',
                    'serviceAccount': 'default',
                    'nodeName': 'bzorp',
                    'securityContext': {},
                    'schedulerName': 'default-scheduler',
                    'tolerations': [
                        {'key': 'node.kubernetes.io/not-ready', 'operator': 'Exists', 'effect': 'NoExecute', 'tolerationSeconds': 300},
                        {'key': 'node.kubernetes.io/unreachable', 'operator': 'Exists', 'effect': 'NoExecute', 'tolerationSeconds': 300}
                    ],
                    'priority': 0,
                    'enableServiceLinks': True,
                    'preemptionPolicy': 'PreemptLowerPriority'
                },
                'status': {
                    'phase': 'Running',
                    'conditions': [
                        {'type': 'Initialized', 'status': 'True', 'lastProbeTime': None, 'lastTransitionTime': '2022-04-20T08:08:37Z'},
                        {'type': 'Ready', 'status': 'True', 'lastProbeTime': None, 'lastTransitionTime': '2022-11-01T12:04:11Z'},
                        {'type': 'ContainersReady', 'status': 'True', 'lastProbeTime': None, 'lastTransitionTime': '2022-11-01T12:04:11Z'},
                        {'type': 'PodScheduled', 'status': 'True', 'lastProbeTime': None, 'lastTransitionTime': '2022-04-20T08:08:37Z'}
                    ],
                    'hostIP': '192.168.78.135',
                    'podIP': '10.1.71.91',
                    'podIPs': [{'ip': '10.1.71.91'}],
                    'startTime': '2022-04-20T08:08:37Z',
                    'containerStatuses': [
                        {
                            'name': 'nginx',
                            'state': {'running': {'startedAt': '2022-11-01T12:04:11Z'}},
                            'lastState': {
                                'terminated': {
                                    'exitCode': 255,
                                    'reason': 'Unknown',
                                    'startedAt': '2022-11-01T10:04:14Z',
                                    'finishedAt': '2022-11-01T12:03:41Z',
                                    'containerID': 'containerd://b010da89c145a03309f43af342774f7c6299c51e4758a8d5416e066d250453b4'
                                }
                            },
                            'ready': True,
                            'restartCount': 67,
                            'image': 'docker.io/library/nginx:latest',
                            'imageID': 'docker.io/library/nginx@sha256:943c25b4b66b332184d5ba6bb18234273551593016c0e0ae906bab111548239f',
                            'containerID': 'containerd://5d297407aef804c35b62734e9b9d03da1f1145c5b45ac12ec8cc216afafef48b',
                            'started': True
                        }
                    ],
                    'qosClass': 'BestEffort'
                }
            }
        ]
    }

    yield mocker.patch('kubernetes.client.rest.RESTClientObject.request', return_value=mocker.Mock(**{
        'status_code': 200,
        'data': json.dumps(response_json)
    }))


def test_kubernetes_api_enrichment_can_be_instantiated():
    assert KubernetesAPIEnrichment() is not None


def test_message_from_var_log_pods_is_enriched(minimal_config, mock_kube_config, mock_api_response_nginx):
    p = KubernetesAPIEnrichment()
    p.init(minimal_config)

    msg = LogMessage("This is the MESSAGE payload")

    #
    # /var/log/pods sets (namespace_name, pod_name, pod_uuid, container_name)
    #
    # sample filename:
    #     /var/log/pods/default_nginx-85b98978db-2dpdh_d7a782cf-cfaa-45e6-8f22-950b2df5bf82/nginx/66.log
    msg['myk8s.namespace_name'] = 'default'
    msg['myk8s.pod_name'] = 'nginx-85b98978db-2dpdh'
    msg['myk8s.pod_uuid'] = 'd7a782cf-cfaa-45e6-8f22-950b2df5bf82'
    msg['myk8s.container_name'] = 'nginx'
    p.parse(msg)

    # fields extracted from the filename stays intact
    assert msg['myk8s.namespace_name'] == b'default'
    assert msg['myk8s.pod_name'] == b'nginx-85b98978db-2dpdh'
    assert msg['myk8s.pod_uuid'] == b'd7a782cf-cfaa-45e6-8f22-950b2df5bf82'
    assert msg['myk8s.container_name'] == b'nginx'

    # fields extracted from the metadata
    assert msg['myk8s.labels.app'] == b'nginx'
    assert msg['myk8s.annotations.cni.projectcalico.org/podIP'] == b'10.1.71.91/32'
    assert msg['myk8s.container_image'] == b'docker.io/library/nginx:latest'


def test_message_from_var_log_containers_is_enriched(minimal_config, mock_kube_config, mock_api_response_nginx):
    p = KubernetesAPIEnrichment()
    p.init(minimal_config)

    msg = LogMessage("This is the MESSAGE payload")

    #
    # /var/log/containers sets (pod_name, namespace_name, container_name, container_id)
    #
    # sample filename:
    # /var/log/containers/nginx-85b98978db-2dpdh_default_nginx-b010da89c145a03309f43af342774f7c6299c51e4758a8d5416e066d250453b4.log
    msg['myk8s.pod_name'] = 'nginx-85b98978db-2dpdh'
    msg['myk8s.namespace_name'] = 'default'
    msg['myk8s.container_name'] = 'nginx'
    msg['myk8s.container_id'] = 'b010da89c145a03309f43af342774f7c6299c51e4758a8d5416e066d250453b4'
    p.parse(msg)

    # fields extracted from the filename stays intact
    assert msg['myk8s.pod_name'] == b'nginx-85b98978db-2dpdh'
    assert msg['myk8s.namespace_name'] == b'default'
    assert msg['myk8s.container_name'] == b'nginx'
    assert msg['myk8s.container_id'] == b'b010da89c145a03309f43af342774f7c6299c51e4758a8d5416e066d250453b4'

    # fields extracted from the metadata
    assert msg['myk8s.labels.app'] == b'nginx'
    assert msg['myk8s.annotations.cni.projectcalico.org/podIP'] == b'10.1.71.91/32'
    assert msg['myk8s.container_image'] == b'docker.io/library/nginx:latest'
    assert msg['myk8s.pod_uuid'] == b'd7a782cf-cfaa-45e6-8f22-950b2df5bf82'


def test_key_delimiter_would_replace_dot(minimal_config, mock_kube_config, mock_api_response_nginx):
    p = KubernetesAPIEnrichment()
    minimal_config['key_delimiter'] = '~'
    minimal_config['prefix'] = 'k8s.'
    p.init(minimal_config)

    msg = LogMessage("This is the MESSAGE payload")

    msg['k8s.pod_name'] = 'nginx-85b98978db-2dpdh'
    msg['k8s.namespace_name'] = 'default'
    p.parse(msg)

    # fields extracted from the metadata
    assert msg['k8s.labels~app'] == b'nginx'
    assert msg['k8s.annotations~cni.projectcalico.org/podIP'] == b'10.1.71.91/32'
    assert msg['k8s.container_image'] == b'docker.io/library/nginx:latest'
    assert msg['k8s.pod_uuid'] == b'd7a782cf-cfaa-45e6-8f22-950b2df5bf82'



#!/bin/env python3

import json
import sys
import urllib.parse
import urllib.request
from time import sleep


def get_workflow_runs(workflow):
    url = "https://api.github.com/repos/syslog-ng/syslog-ng/actions/workflows/{}.yml/runs?branch=master".format(workflow)
    request = urllib.request.Request(url, None, {})
    try:
        with urllib.request.urlopen(request) as response:
            binary_response = response.read()
    except urllib.error.HTTPError:
        print('"' + workflow + '" is not a valid workflow. See .github/workflows/')
        sys.exit(1)

    runs = json.loads(binary_response.decode())["workflow_runs"]
    if len(runs) == 0:
        print('No runs have been executed of "' + workflow + '" yet.')
        sys.exit(1)

    return runs


def main():
    workflow = sys.argv[1]
    while True:
        runs = get_workflow_runs(workflow)
        not_pr_runs = filter(lambda run: run["event"] != "pull_request", runs)
        latest_run = max(not_pr_runs, key=lambda run: run["updated_at"])

        if latest_run["status"] != "completed":
            print("Job is still running. Trying again in 10 minutes.")
            sleep(10 * 60)
            continue

        result = latest_run["conclusion"]

        print('"' + latest_run["name"] + '" job finished.')
        print("Date: " + latest_run["updated_at"])
        print("URL: " + latest_run["html_url"])
        print("Result: " + result)

        if result == "success":
            sys.exit(0)
        else:
            sys.exit(1)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2020 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from src.syslog_ng_config.statements.parsers.db_parser import DBParserConfig


def test_db_parser(config, syslog_ng):
    generator_source = config.create_example_msg_generator_source(
        num=1, template=config.stringify("some number: 5"),
        values="PROGRAM => 'program_name'",
    )

    patterndb_config = DBParserConfig("program_name", [{"class": "patterndb", "rule": "some number: @NUMBER:foo@"}])
    db_parser = config.create_db_parser(patterndb_config)

    file_destination = config.create_file_destination(file_name="output.log", template=config.stringify('foo=$foo class=${.classifier.class}\n'))
    config.create_logpath(statements=[generator_source, db_parser, file_destination])

    syslog_ng.start(config)
    assert file_destination.read_log().strip() == "foo=5 class=patterndb"



#############################################################################
# Copyright (c) 2007-2016 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
# syslog-ng process control

import os, sys, signal, traceback, time, errno, re

from globals import *
from log import *
import messagegen

syslogng_pid = 0


def start_syslogng(conf, keep_persist=False, verbose=False):
    global syslogng_pid

    os.system('rm -rf test-*.log test-*.lgs test-*.db wildcard/* log-file')
    if not keep_persist:
        os.system('rm -f syslog-ng.persist')

    if not logstore_store_supported:
        conf = re.sub('logstore\(.*\);', '', conf)

    f = open('test.conf', 'w')
    f.write(conf)
    f.close()

    if verbose:
        verbose_opt = '-edv'
    else:
        verbose_opt = '-e'

    syslogng_pid = os.fork()
    if syslogng_pid == 0:
        os.putenv("RANDFILE", "rnd")
        module_path = get_module_path()
        rc = os.execl(get_syslog_ng_binary(), get_syslog_ng_binary(), '-f', 'test.conf', '--fd-limit', '1024', '-F', verbose_opt, '-p', 'syslog-ng.pid', '-R', 'syslog-ng.persist', '--no-caps', '--enable-core', '--seed', '--module-path', module_path)
        sys.exit(rc)
    time.sleep(5)
    print_user("Syslog-ng started")
    return True

def stop_syslogng():
    global syslogng_pid

    if syslogng_pid == 0:
        return True

    try:
        os.kill(syslogng_pid, signal.SIGTERM)
    except OSError:
        pass
    try:
        try:
            (pid, rc) = os.waitpid(syslogng_pid, 0)
        except OSError:
            raise
    finally:
        syslogng_pid = 0
    print_user("syslog-ng stopped")
    if rc == 0:
        return True
    print_user("syslog-ng exited with a non-zero value (%d)" % rc)
    return False

def flush_files(settle_time=3):
    global syslogng_pid

    if syslogng_pid == 0 or not messagegen.need_to_flush:
        return True

    print_user("waiting for syslog-ng to settle down before SIGHUP (%d secs)" % settle_time)
    # allow syslog-ng to settle
    time.sleep(settle_time)

    # sendMessages waits between etaps, so we assume that syslog-ng has
    # already received/processed everything we've sent to it. Go ahead send
    # a HUP signal.
    try:
        print_user("Sending syslog-ng the HUP signal (pid: %d)" % syslogng_pid)
        os.kill(syslogng_pid, signal.SIGHUP)
    except OSError:
        print_user("Error sending HUP signal to syslog-ng")
        raise
    # allow syslog-ng to perform config reload & file flush
    print_user("waiting for syslog-ng to process SIGHUP (%d secs)" % 2)
    time.sleep(2)
    messagegen.need_to_flush = False


def readpidfile(pidfile):
    f = open(pidfile, 'r')
    pid = f.read()
    f.close()
    return int(pid.strip())



#############################################################################
# Copyright (c) 2015-2016 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

from __future__ import absolute_import, print_function
from .. import templatelexer
from .test_lexer import TestLexer


class TestTemplateLexer(TestLexer):

    def _construct_lexer(self):
        return templatelexer.TemplateLexer()

    def test_template_literals_are_returned_as_literal_tokens(self):
        self._lexer.input("foobar barfoo")
        self._assert_next_token_equals("LITERAL", value="foobar barfoo", pos=0)

    def test_template_unbraced_macros_are_returned_as_macros(self):
        self._lexer.input("$MSG")
        self._assert_next_token_equals("MACRO", value="$MSG", pos=0)

    def test_template_braced_macros_are_returned_as_macros(self):
        self._lexer.input("${MSG}")
        self._assert_next_token_equals("MACRO", value="${MSG}", pos=0)

    def test_template_simple_template_functions_are_returned_as_template_funcs(self):
        self._lexer.input("$(echo $MSG)")
        self._assert_next_token_equals("TEMPLATE_FUNC", value="$(echo $MSG)", pos=0)

    def test_template_complex_template_functions_are_returned_as_template_funcs(self):
        self._lexer.input("$(echo $(echo $MSG))")
        self._assert_next_token_equals("TEMPLATE_FUNC", value="$(echo $(echo $MSG))", pos=0)

    def test_template_even_more_complex_template_functions_are_returned_as_template_funcs(self):
        self._lexer.input("$(grep ('$COUNT' == 5) $MSG)")
        self._assert_next_token_equals("TEMPLATE_FUNC", value="$(grep ('$COUNT' == 5) $MSG)", pos=0)

    def test_stray_dollar_at_the_end_of_input_is_returned_as_a_literal_token_with_partial_set(self):
        self._lexer.input("$")
        self._assert_next_token_equals("LITERAL", value="$", pos=0, partial=True)

    def test_stray_dollar_brace_at_the_end_of_input_is_returned_as_a_literal_token_with_partial_set(self):
        self._lexer.input("${")
        self._assert_next_token_equals("LITERAL", value="${", pos=0, partial=True)

    def test_stray_dollar_paren_at_the_end_of_input_is_returned_as_a_literal_token_with_partial_set(self):
        self._lexer.input("$(")
        self._assert_next_token_equals("LITERAL", value="$(", pos=0, partial=True)

    def test_unfinished_template_function_at_the_end_of_input_is_returned_as_a_literal_token_with_partial_set(self):
        self._lexer.input("$(echo")
        self._assert_next_token_equals("LITERAL", value="$(echo", pos=0, partial=True)

        self._lexer.input("$(echo $(echo $MSG)")
        self._assert_next_token_equals("LITERAL", value="$(echo $(echo $MSG)", pos=0, partial=True)

    def test_string_within_parens_returned_intact(self):
        self._lexer.input("""$("foobar")""")
        self._assert_next_token_equals("TEMPLATE_FUNC", value='$("foobar")', pos=0)

        self._lexer.input("""$("foo\\"bar")""")
        self._assert_next_token_equals("TEMPLATE_FUNC", value="""$("foo\\"bar")""", pos=0)
        self._lexer.input("""$("foo\\")bar")""")
        self._assert_next_token_equals("TEMPLATE_FUNC", value="""$("foo\\")bar")""", pos=0)

        self._lexer.input("""$('foobar')""")
        self._assert_next_token_equals("TEMPLATE_FUNC", value="$('foobar')", pos=0)

        self._lexer.input("""$('foo"bar')""")
        self._assert_next_token_equals("TEMPLATE_FUNC", value="""$('foo"bar')""", pos=0)

    def test_quoted_parens_dont_close_the_template_function(self):
        self._lexer.input("$(echo ')'")
        self._assert_next_token_equals("LITERAL", value="$(echo ')'", pos=0, partial=True)

        self._lexer.input("$(echo ')')")
        self._assert_next_token_equals("TEMPLATE_FUNC", value="$(echo ')')", pos=0)

    def test_quotes_outside_template_functions_are_returned_literally(self):
        self._lexer.input("'$(echo $MSG)'")
        self._assert_next_token_equals("LITERAL", value="'", pos=0)
        self._assert_next_token_equals("TEMPLATE_FUNC", value="$(echo $MSG)", pos=1)
        self._assert_next_token_equals("LITERAL", value="'", pos=13)

    def test_braced_macro_splits_literals_in_half(self):
        self._lexer.input("foobar${MSG}barfoo")
        self._assert_next_token_equals("LITERAL", value="foobar", pos=0)
        self._assert_next_token_equals("MACRO", value="${MSG}", pos=6)
        self._assert_next_token_equals("LITERAL", value="barfoo", pos=12)

    def test_unbraced_macros_split_literals_in_half(self):
        self._lexer.input("foobar $MSG barfoo")
        self._assert_next_token_equals("LITERAL", value="foobar ", pos=0)
        self._assert_next_token_equals("MACRO", value="$MSG", pos=7)
        self._assert_next_token_equals("LITERAL", value=" barfoo", pos=11)

    def test_template_function_splits_literals_in_half(self):
        self._lexer.input("foobar$(echo $MSG)barfoo")
        self._assert_next_token_equals("LITERAL", value="foobar", pos=0)
        self._assert_next_token_equals("TEMPLATE_FUNC", value="$(echo $MSG)", pos=6)
        self._assert_next_token_equals("LITERAL", value="barfoo", pos=18)



#############################################################################
# Copyright (c) 2022 One Identity
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

import logging
from abc import ABC, abstractmethod
from pathlib import Path

from cdn import CDN
from remote_storage_synchronizer import RemoteStorageSynchronizer


class Indexer(ABC):
    def __init__(
        self,
        incoming_remote_storage_synchronizer: RemoteStorageSynchronizer,
        incoming_sub_dir: Path,
        indexed_remote_storage_synchronizer: RemoteStorageSynchronizer,
        indexed_sub_dir: Path,
        cdn: CDN,
    ) -> None:
        self.__incoming_remote_storage_synchronizer = incoming_remote_storage_synchronizer
        self.__indexed_remote_storage_synchronizer = indexed_remote_storage_synchronizer
        self.__incoming_remote_storage_synchronizer.set_sub_dir(incoming_sub_dir)
        self.__indexed_remote_storage_synchronizer.set_sub_dir(indexed_sub_dir)

        self.__cdn = cdn
        self.__logger = Indexer.__create_logger()

    def __sync_from_remote(self) -> None:
        self.__incoming_remote_storage_synchronizer.sync_from_remote()
        self.__indexed_remote_storage_synchronizer.sync_from_remote()

    @abstractmethod
    def _prepare_indexed_dir(self, incoming_dir: Path, indexed_dir: Path) -> None:
        pass

    @abstractmethod
    def _index_pkgs(self, indexed_dir: Path) -> None:
        pass

    @abstractmethod
    def _sign_pkgs(self, indexed_dir: Path) -> None:
        pass

    def __create_snapshot_of_indexed(self) -> None:
        self.__indexed_remote_storage_synchronizer.create_snapshot_of_remote()

    def __sync_to_remote(self) -> None:
        self.__incoming_remote_storage_synchronizer.sync_to_remote()
        self.__indexed_remote_storage_synchronizer.sync_to_remote()

    def __refresh_cdn_cache(self) -> None:
        path = Path(self.__indexed_remote_storage_synchronizer.remote_dir.working_dir, "*")
        self.__cdn.refresh_cache(path)

    def index(self) -> None:
        incoming_dir = self.__incoming_remote_storage_synchronizer.local_dir.working_dir
        indexed_dir = self.__indexed_remote_storage_synchronizer.local_dir.working_dir

        self.__sync_from_remote()

        self._prepare_indexed_dir(incoming_dir, indexed_dir)
        self._index_pkgs(indexed_dir)
        self._sign_pkgs(indexed_dir)

        self.__create_snapshot_of_indexed()
        self.__sync_to_remote()
        self.__refresh_cdn_cache()

    def flush_cdn_cache(self) -> None:
        self.__refresh_cdn_cache()

    @staticmethod
    def __create_logger() -> logging.Logger:
        logger = logging.getLogger("Indexer")
        logger.setLevel(logging.INFO)
        return logger

    def _log_info(self, message: str, **kwargs: str) -> None:
        log = message
        if len(kwargs) > 0:
            log += "\t{}".format(kwargs)
        self.__logger.info(log)



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2018 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import pytest


@pytest.mark.parametrize(
    "follow_freq,expected", [
        (1, True),
        (1.0, True),
        (0.1, True),
        (0, True),
        (0.0, True),
        (-1, False),
        (-1.0, False),
        (-0.1, False),
    ],
)
def test_follow_freq_value(config, syslog_ng, follow_freq, expected):
    raw_config = '@version: {}\nsource s_file {{ file("input.log" follow-freq({})); }};'.format(config.get_version(), follow_freq)
    config.set_raw_config(raw_config)

    if expected is True:
        syslog_ng.start(config)
    else:
        with pytest.raises(Exception):
            syslog_ng.start(config)



#############################################################################
# Copyright (c) 2015 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

from globals import *
from log import *
from messagegen import *
from messagecheck import *
from control import flush_files, stop_syslogng
import os

config = """@version: %(syslog_ng_version)s

@module mod-python use-virtualenv(no)

options { keep-hostname(yes); };

source s_int { internal(); };
source s_tcp { tcp(port(%(port_number)d)); };

destination d_python {
    python(class(sngtestmod.DestTest)
           value-pairs(key('MSG') pair('HOST', 'bzorp') pair('DATE', '$ISODATE') key('MSGHDR')));
};

log { source(s_tcp); destination(d_python); };

python {

from syslogng import LogParser

class MyParser(LogParser):
  def init(self, options):
      return True

  def deinit(self):
      return True

  def parse(self, msg):
      msg['FOOBAR'] = msg['MSG']
      return True

};

log {
  source(s_tcp);
  parser {
      python(class("MyParser"));
  };
  destination { file("test-python-parser.log" template("$ISODATE $HOST $MSGHDR$FOOBAR\n")); };
};

""" % locals()


def get_source_dir():
    return os.path.abspath(os.path.dirname(__file__))

def check_env():

    if not has_module('mod-python'):
        print('Python module is not available, skipping Python test')
        return False

    src_dir = get_source_dir()
    print(src_dir)
    if 'PYTHONPATH' not in os.environ or src_dir not in os.environ['PYTHONPATH']:
        os.environ['PYTHONPATH'] = os.environ.get('PYTHONPATH', '') + ':' + src_dir

    print('Python module found, proceeding to Python tests')
    return True


def test_python():

    messages = (
        'python1',
        'python2'
    )
    s = SocketSender(AF_INET, ('localhost', port_number), dgram=0)

    expected = []
    for msg in messages:
        expected.extend(s.sendMessages(msg, pri=7))
    flush_files(2)
    stopped = stop_syslogng()
    if not stopped or not check_file_expected('test-python', expected, settle_time=2):
        return False
    return True

def test_python_parser():

    messages = (
        'python_parser1',
        'python_parser2'
    )
    s = SocketSender(AF_INET, ('localhost', port_number), dgram=0)

    expected = []
    for msg in messages:
        expected.extend(s.sendMessages(msg, pri=7))
    flush_files(2)
    stopped = stop_syslogng()
    if not stopped or not check_file_expected('test-python-parser', expected, settle_time=2):
        return False
    return True



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2018 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from src.syslog_ng.console_log_reader import ConsoleLogReader
from src.syslog_ng.syslog_ng_cli import SyslogNgCli


class SyslogNg(object):
    def __init__(self, instance_paths, testcase_parameters, teardown):
        self.instance_paths = instance_paths
        self.teardown = teardown
        self.__syslog_ng_cli = SyslogNgCli(instance_paths, testcase_parameters, self.teardown)

    def start(self, config, stderr=True, debug=True, trace=True, verbose=True, startup_debug=True, no_caps=True, config_path=None, persist_path=None, pid_path=None, control_socket_path=None):
        return self.__syslog_ng_cli.start(config, stderr, debug, trace, verbose, startup_debug, no_caps, config_path, persist_path, pid_path, control_socket_path)

    def stop(self, unexpected_messages=None):
        self.__syslog_ng_cli.stop(unexpected_messages)

    def reload(self, config):
        self.__syslog_ng_cli.reload(config)

    def restart(self, config):
        self.__syslog_ng_cli.stop()
        self.__syslog_ng_cli.start(config)

    def get_version(self):
        return self.__syslog_ng_cli.get_version()

    def is_process_running(self):
        return self.__syslog_ng_cli.is_process_running()

    def wait_for_messages_in_console_log(self, expected_messages):
        assert issubclass(type(expected_messages), list)
        console_log_reader = ConsoleLogReader(self.instance_paths, self.teardown)
        return console_log_reader.wait_for_messages_in_console_log(expected_messages)

    def wait_for_message_in_console_log(self, expected_message):
        return self.wait_for_messages_in_console_log([expected_message])



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2023 Attila Szakacs
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################


def test_python_custom_options(config, syslog_ng):
    raw_config = r"""
python {
import syslogng

def assert_options(options):
    assert options["string"] == "example_string", "Actual: {}".format(options["string"])
    assert options["true"] == True, "Actual: {}".format(options["true"])
    assert options["false"] == False, "Actual: {}".format(options["false"])
    assert options["long"] == 123456789, "Actual: {}".format(options["long"])
    assert options["double"] == 123.456789, "Actual: {}".format(options["double"])
    assert options["string_list"] == ["string1", "string2"], "Actual: {}".format(options["string_list"])
    assert isinstance(options["template"], syslogng.LogTemplate), "Actual: {}".format(type(options["template"]))
    assert str(options["template"]) == "${template}", "Actual: {}".format(str(options["template"]))


class TestSource(syslogng.LogSource):
  def init(self, options):
    assert_options(options)
    return True


class TestFetcher(syslogng.LogFetcher):
  def init(self, options):
    assert_options(options)
    return True


class TestDestination(syslogng.LogDestination):
  def init(self, options):
    assert_options(options)
    return True


class TestParser(syslogng.LogParser):
  def init(self, options):
    assert_options(options)
    return True


class TestHttpHeader:
  def __init__(self, options):
    assert_options(options)

  def get_headers():
    pass

  def on_http_response_received():
    pass


def test_confgen(options):
  return '''
    python(
      class("TestDestination")
      options(
        "string" => {}
        "true" => {}
        "false" => {}
        "long" => {}
        "double" => {}
        "string-list" => [{}]
        "template" => LogTemplate({})
      )
      persist-name("custom-persist-name")
    );
  '''.format(options["string"], options["true"], options["false"],
             options["long"], options["double"], options["string_list"],
             options["template"])


syslogng.register_config_generator(context="destination", name="test_confgen",
                                   config_generator=test_confgen)
};

log {
  source {
    python(
      class("TestSource")
      options(
        "string" => "example_string"
        "true" => True
        "false" => False
        "long" => 123456789
        "double" => 123.456789
        "string-list" => ["string1", "string2"]
        "template" => LogTemplate("${template}")
      )
    );
    python-fetcher(
      class("TestFetcher")
      options(
        "string" => "example_string"
        "true" => True
        "false" => False
        "long" => 123456789
        "double" => 123.456789
        "string-list" => ["string1", "string2"]
        "template" => LogTemplate("${template}")
      )
    );
  };
  parser {
    python(
      class("TestParser")
      options(
        "string" => "example_string"
        "true" => True
        "false" => False
        "long" => 123456789
        "double" => 123.456789
        "string-list" => ["string1", "string2"]
        "template" => LogTemplate("${template}")
      )
    );
  };
  destination {
    python(
      class("TestDestination")
      options(
        "string" => "example_string"
        "true" => True
        "false" => False
        "long" => 123456789
        "double" => 123.456789
        "string-list" => ["string1", "string2"]
        "template" => LogTemplate("${template}")
      )
    );
    test-confgen(
      string("example_string")
      true(yes)
      false(no)
      long(123456789)
      double(123.456789)
      string-list("string1", "string2")
      template("${template}")
    );
  };
};
"""
    raw_config = "@version: {}\n".format(config.get_version()) + raw_config
    config.set_raw_config(raw_config)

    syslog_ng.start(config)



#############################################################################
# Copyright (c) 2017 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

from globals import *
from log import *
from messagegen import *
from messagecheck import *
from control import flush_files, stop_syslogng
import os

config = """@version: %(syslog_ng_version)s

options { keep-hostname(yes); };

source s_int { internal(); };
source s_tcp { tcp(port(%(port_number)d)); };

parser p_map {
  map-value-pairs(key("MSG*" rekey(add-prefix("foo."))));
};

log {
  source(s_tcp);
  parser(p_map);
  destination { file("test-map-value-pairs.log" template("$ISODATE $HOST $MSGHDR${foo.MSG}\n")); };
};

""" % locals()


def get_source_dir():
    return os.path.abspath(os.path.dirname(__file__))

def test_map_value_pairs():

    messages = (
        'map_value_pairs1',
        'map_value_pairs2'
    )
    s = SocketSender(AF_INET, ('localhost', port_number), dgram=0)

    expected = []
    for msg in messages:
        expected.extend(s.sendMessages(msg, pri=7))
    stopped = stop_syslogng()
    if not stopped or not check_file_expected('test-map-value-pairs', expected, settle_time=2):
        return False
    return True



#############################################################################
# Copyright (c) 2022 One Identity
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

from hashlib import md5
from pathlib import Path
from typing import List, Optional

from azure.storage.blob import BlobClient, ContainerClient

from .remote_storage_synchronizer import FileSyncState, RemoteStorageSynchronizer

DEFAULT_ROOT_DIR = Path("/tmp/azure_container_synchronizer")


class AzureContainerSynchronizer(RemoteStorageSynchronizer):
    """
    A `RemoteStorageSynchronizer` implementation that can connect to an Azure Blob Container Storage instance.

    Example config:

    ```yaml
    vendor: "azure"
    incoming:
      all:
        storage-name: "incoming"
        connection-string: "secret1"
    indexed:
      stable:
        storage-name: "indexed"
        connection-string: "secret2"
      nightly:
        storage-name: "indexed"
        connection-string: "secret3"
    ```
    """

    def __init__(self, connection_string: str, storage_name: str) -> None:
        self.__client = ContainerClient.from_connection_string(conn_str=connection_string, container_name=storage_name)
        self.__remote_files_cache: Optional[List[dict]] = None
        super().__init__(
            remote_root_dir=Path(""),
            local_root_dir=Path(DEFAULT_ROOT_DIR, storage_name),
        )

    @staticmethod
    def get_config_keyword() -> str:
        return "azure"

    @staticmethod
    def from_config(cfg: dict) -> RemoteStorageSynchronizer:
        return AzureContainerSynchronizer(
            connection_string=cfg["connection-string"],
            storage_name=cfg["storage-name"],
        )

    @property
    def local_files(self) -> List[Path]:
        dirs_and_files = list(self.local_dir.working_dir.rglob("*"))
        return list(filter(lambda path: path.is_file(), dirs_and_files))

    @property
    def remote_files(self) -> List[dict]:
        if self.__remote_files_cache is not None:
            return self.__remote_files_cache

        file_name_prefix = "{}/".format(self.remote_dir.working_dir)
        self.__remote_files_cache = [dict(blob) for blob in self.__client.list_blobs(name_starts_with=file_name_prefix)]

        return self.__remote_files_cache

    def __download_file(self, relative_file_path: str) -> None:
        download_path = Path(self.local_dir.root_dir, relative_file_path).resolve()

        self._log_info(
            "Downloading file.",
            remote_path=relative_file_path,
            local_path=str(download_path),
        )

        download_path.parent.mkdir(parents=True, exist_ok=True)
        with download_path.open("wb") as downloaded_blob:
            blob_data = self.__client.download_blob(relative_file_path)
            blob_data.readinto(downloaded_blob)

    def __upload_file(self, relative_file_path: str) -> None:
        local_path = Path(self.local_dir.root_dir, relative_file_path)

        self._log_info(
            "Uploading file.",
            local_path=str(local_path),
            remote_path=relative_file_path,
        )

        with local_path.open("rb") as local_file_data:
            self.__client.upload_blob(relative_file_path, local_file_data, overwrite=True)

    def __delete_local_file(self, relative_file_path: str) -> None:
        local_file_path = Path(self.local_dir.root_dir, relative_file_path).resolve()
        self._log_info("Deleting local file.", local_path=str(local_file_path))
        local_file_path.unlink()

    def __delete_remote_file(self, relative_file_path: str) -> None:
        self._log_info("Deleting remote file.", remote_path=relative_file_path)
        self.__client.delete_blob(relative_file_path, delete_snapshots="include")

    def sync_from_remote(self) -> None:
        self._log_info(
            "Syncing content from remote.",
            remote_workdir=str(self.remote_dir.working_dir),
            local_workdir=str(self.local_dir.working_dir),
        )
        for file in self.__all_files:
            sync_state = self.__get_file_sync_state(file)
            if sync_state == FileSyncState.IN_SYNC:
                continue
            if sync_state == FileSyncState.DIFFERENT or sync_state == FileSyncState.NOT_IN_LOCAL:
                self.__download_file(file)
                continue
            if sync_state == FileSyncState.NOT_IN_REMOTE:
                self.__delete_local_file(file)
                continue
            raise NotImplementedError("Unexpected FileSyncState: {}".format(sync_state))
        self._log_info(
            "Successfully synced remote content.",
            remote_workdir=str(self.remote_dir.working_dir),
            local_workdir=str(self.local_dir.working_dir),
        )

    def sync_to_remote(self) -> None:
        self._log_info(
            "Syncing content to remote.",
            local_workdir=str(self.local_dir.working_dir),
            remote_workdir=str(self.remote_dir.working_dir),
        )
        for file in self.__all_files:
            sync_state = self.__get_file_sync_state(file)
            if sync_state == FileSyncState.IN_SYNC:
                continue
            if sync_state == FileSyncState.DIFFERENT or sync_state == FileSyncState.NOT_IN_REMOTE:
                self.__upload_file(file)
                continue
            if sync_state == FileSyncState.NOT_IN_LOCAL:
                self.__delete_remote_file(file)
                continue
            raise NotImplementedError("Unexpected FileSyncState: {}".format(sync_state))
        self._log_info(
            "Successfully synced local content.",
            remote_workdir=str(self.remote_dir.working_dir),
            local_workdir=str(self.local_dir.working_dir),
        )
        self.__invalidate_remote_files_cache()

    def create_snapshot_of_remote(self) -> None:
        self._log_info("Creating snapshot of the remote container.")
        for file in self.remote_files:
            blob_client: BlobClient = self.__client.get_blob_client(file["name"])
            snapshot_properties = blob_client.create_snapshot()
            self._log_debug(
                "Successfully created snapshot of remote file.",
                remote_path=self.__get_relative_file_path_for_remote_file(file),
                snapshot_properties=snapshot_properties,
            )

    def __get_md5_of_remote_file(self, relative_file_path: str) -> bytearray:
        for file in self.remote_files:
            if file["name"] == relative_file_path:
                return file["content_settings"]["content_md5"]
        raise FileNotFoundError

    def __get_md5_of_local_file(self, relative_file_path: str) -> bytes:
        file = Path(self.local_dir.root_dir, relative_file_path)
        return md5(file.read_bytes()).digest()

    def __get_file_sync_state(self, relative_file_path: str) -> FileSyncState:
        try:
            remote_md5 = self.__get_md5_of_remote_file(relative_file_path)
        except FileNotFoundError:
            self._log_debug(
                "Local file is not available remotely.",
                local_path=str(Path(self.local_dir.root_dir, relative_file_path)),
                unavailable_remote_path=str(Path(self.remote_dir.root_dir, relative_file_path)),
            )
            return FileSyncState.NOT_IN_REMOTE

        try:
            local_md5 = self.__get_md5_of_local_file(relative_file_path)
        except FileNotFoundError:
            self._log_debug(
                "Remote file is not available locally.",
                remote_path=str(Path(self.remote_dir.root_dir, relative_file_path)),
                unavailable_local_path=str(Path(self.local_dir.root_dir, relative_file_path)),
            )
            return FileSyncState.NOT_IN_LOCAL

        if remote_md5 != local_md5:
            self._log_debug(
                "File differs locally and remotely.",
                remote_path=str(Path(self.remote_dir.root_dir, relative_file_path)),
                local_path=str(Path(self.local_dir.root_dir, relative_file_path)),
                remote_md5sum=remote_md5.hex(),
                local_md5sum=local_md5.hex(),
            )
            return FileSyncState.DIFFERENT

        self._log_debug(
            "File is in sync.",
            remote_path=str(Path(self.remote_dir.root_dir, relative_file_path)),
            local_path=str(Path(self.local_dir.root_dir, relative_file_path)),
            md5sum=remote_md5.hex(),
        )
        return FileSyncState.IN_SYNC

    def __get_relative_file_path_for_local_file(self, file: Path) -> str:
        return str(file.relative_to(self.local_dir.root_dir))

    def __get_relative_file_path_for_remote_file(self, file: dict) -> str:
        return file["name"]

    @property
    def __all_files(self) -> List[str]:
        files = set()
        for local_file in self.local_files:
            files.add(self.__get_relative_file_path_for_local_file(local_file))
        for remote_file in self.remote_files:
            files.add(self.__get_relative_file_path_for_remote_file(remote_file))
        return sorted(files)

    def __invalidate_remote_files_cache(self) -> None:
        self.__remote_files_cache = None

    def _prepare_log(self, message: str, **kwargs: str) -> str:
        log = "[{} :: {}]\t{}".format(self.__client.container_name, str(self.remote_dir.working_dir), message)
        return super()._prepare_log(log, **kwargs)



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2018 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import logging
from pathlib import Path

from src.common.blocking import wait_until_false
from src.common.blocking import wait_until_true
from src.syslog_ng.console_log_reader import ConsoleLogReader
from src.syslog_ng.syslog_ng_executor import SyslogNgExecutor
from src.syslog_ng_ctl.syslog_ng_ctl import SyslogNgCtl

logger = logging.getLogger(__name__)


class SyslogNgCli(object):
    def __init__(self, instance_paths, testcase_parameters, teardown):
        self.__instance_paths = instance_paths
        self.__console_log_reader = ConsoleLogReader(instance_paths, teardown)
        self.__syslog_ng_executor = SyslogNgExecutor(instance_paths)
        self.__syslog_ng_ctl = SyslogNgCtl(instance_paths)
        self.__external_tool = testcase_parameters.get_external_tool()
        self.__process = None
        self.__stderr = None
        self.__debug = None
        self.__trace = None
        self.__verbose = None
        self.__startup_debug = None
        self.__no_caps = None
        self.__config_path = None
        self.__persist_path = None
        self.__pid_path = None
        self.__control_socket_path = None

    # Application commands
    def get_version(self):
        version_output = self.__syslog_ng_executor.run_command(command_short_name="version", command=["--version"])[
            "stdout"
        ]
        for version_output_line in version_output.splitlines():
            if "Config version:" in version_output_line:
                return version_output_line.split()[2]
        raise Exception("Can not parse 'Config version' from ./syslog-ng --version")

    def __syntax_only(self, config_path=None):
        if config_path is None:
            config_path = self.__instance_paths.get_config_path()
        return self.__syslog_ng_executor.run_command(
            command_short_name="syntax_only", command=["--syntax-only", "--cfgfile={}".format(config_path)],
        )

    def __syntax_check(self):
        result = self.__syntax_only()
        if result["exit_code"] != 0:
            logger.error(result["stderr"])
            raise Exception("syslog-ng can not started exit_code={}".format(result["exit_code"]))

    def is_process_running(self):
        return self.__process and self.__process.poll() is None

    def __wait_for_control_socket_alive(self):
        def is_alive(s):
            if not s.is_process_running():
                self.__error_handling("syslog-ng is not running")
            return s.__syslog_ng_ctl.is_control_socket_alive()
        return wait_until_true(is_alive, self)

    def __wait_for_start(self):
        # wait for start and check start result
        if not self.__wait_for_control_socket_alive():
            self.__error_handling("Control socket not alive")
        if not self.__console_log_reader.wait_for_start_message():
            self.__error_handling("Start message not arrived")

    def __start_syslog_ng(self):
        if self.__external_tool:
            self.__process = self.__syslog_ng_executor.run_process_with_external_tool(self.__external_tool, self.__stderr, self.__debug, self.__trace, self.__verbose, self.__startup_debug, self.__no_caps, self.__config_path, self.__persist_path, self.__pid_path, self.__control_socket_path)
        else:
            self.__process = self.__syslog_ng_executor.run_process(self.__stderr, self.__debug, self.__trace, self.__verbose, self.__startup_debug, self.__no_caps, self.__config_path, self.__persist_path, self.__pid_path, self.__control_socket_path)
        if self.__stderr and self.__debug and self.__verbose:
            self.__wait_for_start()

    # Process commands
    def start(self, config, stderr, debug, trace, verbose, startup_debug, no_caps, config_path, persist_path, pid_path, control_socket_path):
        self.set_start_parameters(stderr, debug, trace, verbose, startup_debug, no_caps, config_path, persist_path, pid_path, control_socket_path)
        if self.__process:
            raise Exception("syslog-ng has been already started")

        config.write_config(self.__instance_paths.get_config_path())

        self.__syntax_check()
        self.__start_syslog_ng()

        logger.info("syslog-ng process has been started with PID: {}\n".format(self.__process.pid))

        return self.__process

    def reload(self, config):
        config.write_config(self.__instance_paths.get_config_path())

        # effective reload
        result = self.__syslog_ng_ctl.reload()

        # wait for reload and check reload result
        if result["exit_code"] != 0:
            self.__error_handling("Control socket fails to reload syslog-ng")
        if not self.__wait_for_control_socket_alive():
            self.__error_handling("Control socket not alive")
        if self.__stderr and self.__debug and self.__verbose:
            if not self.__console_log_reader.wait_for_reload_message():
                self.__error_handling("Reload message not arrived")
        logger.info("syslog-ng process has been reloaded with PID: {}\n".format(self.__process.pid))

    def stop(self, unexpected_messages=None):
        if self.__process:
            saved_pid = self.__process.pid
            # effective stop
            result = self.__syslog_ng_ctl.stop()

            # wait for stop and check stop result
            if result["exit_code"] != 0:
                self.__error_handling("Control socket fails to stop syslog-ng")
            if not wait_until_false(self.is_process_running):
                self.__error_handling("syslog-ng did not stop")
            if self.__stderr and self.__debug and self.__verbose:
                if not self.__console_log_reader.wait_for_stop_message():
                    self.__error_handling("Stop message not arrived")
            self.__console_log_reader.check_for_unexpected_messages(unexpected_messages)
            if self.__external_tool == "valgrind":
                self.__console_log_reader.handle_valgrind_log(self.__instance_paths.get_external_tool_output_path(self.__external_tool))
            self.__process = None
            logger.info("syslog-ng process has been stopped with PID: {}\n".format(saved_pid))

    # Helper functions
    def __error_handling(self, error_message):
        self.__console_log_reader.dump_stderr()
        self.__handle_core_file()
        raise Exception(error_message)

    def __handle_core_file(self):
        if not self.is_process_running():
            core_file_found = False
            for core_file in Path(".").glob("*core*"):
                core_file_found = True
                self.__process = None
                self.__syslog_ng_executor.get_backtrace_from_core(core_file=str(core_file))
                core_file.replace(Path(core_file))
            if core_file_found:
                raise Exception("syslog-ng core file was found and processed")
            if self.__process.returncode in [-6, -9, -11]:
                ret_code = self.__process.returncode
                self.__process = None
                raise Exception("syslog-ng process crashed with signal {}".format(ret_code))

    def set_start_parameters(self, stderr, debug, trace, verbose, startup_debug, no_caps, config_path, persist_path, pid_path, control_socket_path):
        self.__stderr = stderr
        self.__debug = debug
        self.__trace = trace
        self.__verbose = verbose
        self.__startup_debug = startup_debug
        self.__no_caps = no_caps
        self.__config_path = config_path
        self.__persist_path = persist_path
        self.__pid_path = pid_path
        self.__control_socket_path = control_socket_path



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2022 One Identity
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import json

import pytest


test_parameters = [
    (
        "<190>Apr 13 14:43:13 mysql-server_auditing: columnstore-1 eff8a68bcd7f,user1,172.18.0.1,32,394,QUERY,syslog_ng,'SELECT * FROM test WHERE 0=1',1146",
        {
            "PRI": "190",
            "DATE": "Apr 13 14:43:13",
            "_mariadb": {
                "username": "user1",
                "syslog_info": "columnstore-1",
                "serverhost": "eff8a68bcd7f",
                "retcode": "1146",
                "queryid": "394",
                "operation": "QUERY",
                "object": "SELECT * FROM test WHERE 0=1",
                "host": "172.18.0.1",
                "database": "syslog_ng",
                "connectionid": "32",
            },
            "MSG": r"columnstore-1 eff8a68bcd7f,user1,172.18.0.1,32,394,QUERY,syslog_ng,'SELECT * FROM test WHERE 0=1',1146",
        },
    ),
    (
        "<190>Sep 14 17:46:51 centos mysql-server_auditing: columnstore-1 centos,root,localhost,11,117,QUERY,loans,'SELECT grade, AVG(loan_amnt) avg,FROM loanstats GROUP BY grade ORDER BY grade',0",
        {
            "PRI": "190",
            "DATE": "Sep 14 17:46:51",
            "_mariadb": {
                "username": "root",
                "syslog_info": "columnstore-1",
                "serverhost": "centos",
                "retcode": "0",
                "queryid": "117",
                "operation": "QUERY",
                "object": "SELECT grade, AVG(loan_amnt) avg,FROM loanstats GROUP BY grade ORDER BY grade",
                "host": "localhost",
                "database": "loans",
                "connectionid": "11",
            },
            "MSG": r"columnstore-1 centos,root,localhost,11,117,QUERY,loans,'SELECT grade, AVG(loan_amnt) avg,FROM loanstats GROUP BY grade ORDER BY grade',0",
        },
    ),
    (
        "<190>Apr 29 13:56:36 mysql-server_auditing: docker eff8a68bcd7f,root,172.18.0.3,13,0,CONNECT,,,0",
        {
            "PRI": "190",
            "DATE": "Apr 29 13:56:36",
            "_mariadb": {
                "username": "root",
                "syslog_info": "docker",
                "serverhost": "eff8a68bcd7f",
                "retcode": "0",
                "queryid": "0",
                "operation": "CONNECT",
                "object": "",
                "host": "172.18.0.3",
                "database": "",
                "connectionid": "13",
            },
            "MSG": r"docker eff8a68bcd7f,root,172.18.0.3,13,0,CONNECT,,,0",
        },
    ),
    (
        "<190>Apr 29 13:56:16 mysql-server_auditing: docker eff8a68bcd7f,user1,172.18.0.3,11,58,QUERY,syslog_ng,'insert into test (date, host, program, message) VALUES(Apr 29 15:55:24, locohost, test-program, foo fighters bar baz)',0",
        {
            "PRI": "190",
            "DATE": "Apr 29 13:56:16",
            "_mariadb": {
                "username": "user1",
                "syslog_info": "docker",
                "serverhost": "eff8a68bcd7f",
                "retcode": "0",
                "queryid": "58",
                "operation": "QUERY",
                "object": "insert into test (date, host, program, message) VALUES(Apr 29 15:55:24, locohost, test-program, foo fighters bar baz)",
                "host": "172.18.0.3",
                "database": "syslog_ng",
                "connectionid": "11",
            },
            "MSG": r"docker eff8a68bcd7f,user1,172.18.0.3,11,58,QUERY,syslog_ng,'insert into test (date, host, program, message) VALUES(Apr 29 15:55:24, locohost, test-program, foo fighters bar baz)',0",
        },
    ),
]


@pytest.mark.parametrize(
    "input_message, expected_kv_pairs",
    test_parameters,
    ids=range(len(test_parameters)),
)
def test_mariadb_audit_parser(config, port_allocator, syslog_ng, input_message, expected_kv_pairs):
    config.add_include("scl.conf")

    network_source = config.create_network_source(ip="localhost", port=port_allocator())
    mariadb_audit_parser = config.create_mariadb_audit_parser()
    file_destination = config.create_file_destination(
        file_name="output.log",
        template=config.stringify("$(format-json --scope everything)" + "\n"),
    )
    config.create_logpath(
        statements=[network_source, mariadb_audit_parser, file_destination],
    )

    syslog_ng.start(config)
    network_source.write_log(input_message)

    output_json = json.loads(file_destination.read_log())

    assert output_json["PRI"] == expected_kv_pairs["PRI"]
    assert output_json["DATE"] == expected_kv_pairs["DATE"]

    assert output_json["_mariadb"]["syslog_info"] == expected_kv_pairs["_mariadb"]["syslog_info"]
    assert output_json["_mariadb"]["serverhost"] == expected_kv_pairs["_mariadb"]["serverhost"]
    assert output_json["_mariadb"]["retcode"] == expected_kv_pairs["_mariadb"]["retcode"]
    assert output_json["_mariadb"]["queryid"] == expected_kv_pairs["_mariadb"]["queryid"]
    assert output_json["_mariadb"]["operation"] == expected_kv_pairs["_mariadb"]["operation"]
    assert output_json["_mariadb"]["object"] == expected_kv_pairs["_mariadb"]["object"]
    assert output_json["_mariadb"]["host"] == expected_kv_pairs["_mariadb"]["host"]
    assert output_json["_mariadb"]["database"] == expected_kv_pairs["_mariadb"]["database"]
    assert output_json["_mariadb"]["connectionid"] == expected_kv_pairs["_mariadb"]["connectionid"]

    assert output_json["MSG"] == expected_kv_pairs["MSG"]



#############################################################################
# Copyright (c) 2015-2016 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

import unittest


class TestLexer(unittest.TestCase):

    def setUp(self):
        self._lexer = self._construct_lexer()
        self._current_token = None

    def _construct_lexer(self):
        raise NotImplementedError

    def _next_token(self):
        self._current_token = self._lexer.token()
        return self._current_token

    def _assert_current_token_type_equals(self, token_type):
        self.assertEqual(self._current_token.type, token_type)

    def _assert_current_token_value_equals(self, value):
        self.assertEqual(self._current_token.value, value)

    def _assert_current_token_pos_equals(self, pos):
        self.assertEqual(self._current_token.lexpos, pos)

    def _assert_current_token_is_partial(self):
        self.assertTrue(self._current_token.partial)

    def _assert_current_token_is_not_partial(self):
        self.assertFalse(self._current_token.partial)

    def _assert_current_token_equals(self, token_type=None, value=None, pos=None, partial=None):
        if token_type is not None:
            self._assert_current_token_type_equals(token_type)
        if value is not None:
            self._assert_current_token_value_equals(value)
        if pos is not None:
            self._assert_current_token_pos_equals(pos)
        if partial is not None:
            if partial:
                self._assert_current_token_is_partial()
            else:
                self._assert_current_token_is_not_partial()

    def _assert_next_token_equals(self, *args, **kwargs):
        self._next_token()
        self._assert_current_token_equals(*args, **kwargs)

    def _assert_next_token_type_equals(self, token_type):
        self._next_token()
        self._assert_current_token_type_equals(token_type)

    def _assert_next_token_is_partial(self):
        self._next_token()
        self._assert_current_token_is_partial()

    def _assert_next_token_is_none(self):
        self.assertIsNone(self._next_token())

    def _assert_next_token_value_equals(self, value):
        self._next_token()
        self._assert_current_token_value_equals(value)
